from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ParentChildPair(TcBaseObj):
    """
    Structure of a pair of parent and child objects for occurrence condition validation.
    
    :var parent: The parent ItemRevision object where to add the child
    :var child: The child ItemRevision or GeneralDesignElement object to be added
    """
    parent: BusinessObject = None
    child: BusinessObject = None


@dataclass
class AddInformation(TcBaseObj):
    """
    Contains information of the object to be added as a new line. When used by Add operation, only one of the
    item/itemRev, element and line will be used.
    
    :var item: The item to be added. It can be null.
    :var itemRev: The item revision to be added. It can be null.
    :var bomView: The BOMView object to be added. It can be null.
    :var element: The General Design Element object to be added. It can be null.
    :var line: The BOMLine object to be pasted as new line. It can be ImanItemLine or GDELine. It can be null.
    :var initialValues: A map to hold initial values of BOMLine properties. Occurrence type can be specified here but
    will be handled specially if used by Add operation.
    """
    item: BusinessObject = None
    itemRev: BusinessObject = None
    bomView: BusinessObject = None
    element: BusinessObject = None
    line: BOMLine = None
    initialValues: BOMLineProperties = None


@dataclass
class AddParam(TcBaseObj):
    """
    The input for adding lines.
    
    :var parent: The parent where the objects are added.
    :var toBeAdded: The information about the objects to be added.
    :var flags: A  bit map of the flags. The lowest bit is reserved for as substitute. If the bit is 1, it means the
    operation will add the object as substitute, otherwise it as normal line. The second lowest bit is for propagating
    transform matrix, when specified as 1, transform matrix will be propagated from source line to the new line. The
    third lowest bit is used for pending cut lines. If set, the line will be removed after being copied.  Other bits
    are not used.
    """
    parent: BOMLine = None
    toBeAdded: List[AddInformation] = ()
    flags: int = 0


@dataclass
class AddResponse(TcBaseObj):
    """
    The response for add operation
    
    :var addedLines: The added lines.
    :var serviceData: The serviceData object that contains partial errors.
    """
    addedLines: List[BOMLine] = ()
    serviceData: ServiceData = None


"""
BOMLine properties and values.
"""
BOMLineProperties = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Fnd0ModelViewProxy, ConfigurationContext
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.Cad._2016_03.StructureManagement import StructureNodeResults
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class FindModelViewsInput(TcBaseObj):
    """
    The set of inputs needed to start a find operation on a given object ('disclosure') or a set of objects
    ('startingScopes') which are structured under the disclosure.
    
    :var searchID: A unique identifier passed by the client in order to identify the find results to continue with when
    calling the continueFindModelViews operation. If a value is not provided, the preference MVFindMinNodeCount will be
    ignored and the find will finish and no followup call to continueFindModelViews can be made.
    :var disclosure: The structure root of the various 'startingScopes'. This structure root must be an ItemRevision or
    Workset (Cpd0Workset).  Required if 'startingScopes' contains more than one object, otherwise it may be left NULL.
    :var startingScopes: A list of one or more structures to search for model view proxy objects. If a single root
    assembly or workset (Cpd0Workset) is given, the entire structure will be searched. If multiple sub-assemblies are
    given, then only those disclosure children under the given sub-assemblies will be searched for proxies. The input
    objects may be any WorkspaceObject sub-type that is expected to own (directly or indirectly) model view proxy
    objects. Most commonly input would be a Workset or WorksetLine, but other ItemRevision or ItemLine objects may be
    given.
    :var configurationContext: The configuration context object used to configure the children of the expanding
    structure by revision rules, effectivity, and variants.
    :var withDisclosureIntents: A list of string values values (such as "Design Review" or "Quality Check" ) to match
    against candidate model view proxies. Only model view proxies that have any of the specified disclosure purpose
    (fnd0DisclosureIntent) attribute values will be returned. If no strings are given or any string is an empty string,
    then all model view proxy objects will be returned.
    :var options: A set of optional flags. Supported options are: "'expandStructureScope'"
    'expandStructureScope': If true, each 'startingScope' will be expanded and their children searched for model views.
    If false, only model view proxies owned by the 'startingScope' objects will be returned. The default is true.
    """
    searchID: str = ''
    disclosure: BusinessObject = None
    startingScopes: List[BusinessObject] = ()
    configurationContext: ConfigurationContext = None
    withDisclosureIntents: List[str] = ()
    options: BoolMap2 = None


@dataclass
class FindModelViewsResponse(TcBaseObj):
    """
    The response contains any found model view proxies (FndModelViewProxy) and the structure location at which they
    were found.
    
    Any errors are returned in the 'serviceData'.
    
    :var modelViewsByStructureNodes: A list of  proxy objects found for various  nodes in the structure - the structure
    nodes may be bomlines, item revisions or design elements depending on the type of structure starting scope and
    disclosure type.
    :var finished: Flag to indicate if the find operation is complete or if there is more content or structure to be
    searched. If false, then the continueFindModelViews operation may be called with the input 'searchID' in order to
    continue to retrieve more model views. If true, the state of the search is cleaned up and the given 'searchID', if
    any, may not be used as input into the continueFindModelViews operation.
    :var structureNodesSearched: Total number of structure nodes ( BomLines or Design Elements) which were so far
    searched for model views. This shows how large the structure is as the search is progressing over multiple calls to
    startFindModelViews and continueFindModelViews.
    :var serviceData: Contains a list of any errors which occurred during the operation.
    """
    modelViewsByStructureNodes: List[StructureNodeResults] = ()
    finished: bool = False
    structureNodesSearched: int = 0
    serviceData: ServiceData = None


@dataclass
class ModelViewProxyReconcileInfo(TcBaseObj):
    """
    This structure contains the reconcile information for each Fnd0ModelViewProxy object found in the
    Fnd0ModelViewPalette. It contains information about the candidate Fnd0ModelViewProxy found, the recommended
    reconciliation action, and a description of the basis on which the candidate Fnd0ModelViewProxy was identified.
    
    :var candidateMVP: The candidate Fnd0ModelViewProxy identified for reconciliation.
    :var reconcileActionEnum: The recommended action for the user to take. Supported values are: 
    - None
    - Replace
    - Remove
    - Resolve Ambiguity
    
    
    :var reconcileAction: The displayable explanation of the action that need to be taken. For example: "Remove Model
    View from Palette"
    :var reconcileReason: The displayable message which describes the basis of candidate identification.  For example,
    "Model View deleted from Owning Model"
    :var candidateStructNodeInfos: A list of 'CandidateModelViewStructNodeInfo' that contains the BOMLine or
    Cpd0DesignElement  on which the candidate Fnd0ModelViewProxy was found.
    """
    candidateMVP: Fnd0ModelViewProxy = None
    reconcileActionEnum: str = ''
    reconcileAction: str = ''
    reconcileReason: str = ''
    candidateStructNodeInfos: List[CandidateModelViewStructNodeInfo] = ()


@dataclass
class NextReconcilePaletteInput(TcBaseObj):
    """
    This structure contains the  information about the  'clientID' that identifies the reconciliation state in the
    server and list of Fnd0ModelViewProxy if they have to be given a higher priority during reconciliation. For
    example, the user may have chosen to apply a Model View on the palette before the palette reconciliation that
    already started was complete.
    
    :var clientID: A unique clientId passed by the client in order to identify the reconciliation of the palette
    between subsequent calls to 'startReconcilePalette 'and' continueReconcilePalette'.
    :var mvProxiesOrMvGroups: List of Fnd0ModelViewProxy  or the  Fnd0ModelViewGroup objects that need to be
    reconciled.  Can be used to do a partial reconciliation or just the ModelViewProxy or Group of interest.
    :var stopReconcile: Flag to stop the reconciliation process.
    """
    clientID: str = ''
    mvProxiesOrMvGroups: List[BusinessObject] = ()
    stopReconcile: bool = False


@dataclass
class PaletteModelViewProxyReconcileInfo(TcBaseObj):
    """
    This structure binds the Fnd0ModelViewProxy in the Fnd0ModelViewPalette to its corresponding reconcile information.
    
    :var paletteMVP: The Fnd0ModelViewProxy on the Fnd0ModelViewPalette that has needs to be reconciled.
    :var reconcileInfos: A list of reconcile information for this Fnd0ModelViewProxy including reasons, actions,
    reconcile candidates, and more. Can be a list if more than one candidate identified.
    """
    paletteMVP: Fnd0ModelViewProxy = None
    reconcileInfos: List[ModelViewProxyReconcileInfo] = ()


@dataclass
class ReconcilePaletteInput(TcBaseObj):
    """
    This structure contains the  information about the  Fnd0ModelViewPalette or the disclosing object that needs to be
    reconciled. It also contains configuration information specified as 'configRecipe'  in the form of BOMLine or
    VisStructureContext object. If BOMLine is specified as 'configRecipe' then the BOMWindow to which the line belongs
    will be used for reconciliation. If the 'configRecipe' is not specified then the configuration information will be
    retrieved from the Fnd0ModelViewPalette if present otherwise the default configuration is used for reconciliation.
    The 'clientID' supplied by the client will be used to identify the reconciliation state in the server to continue
    the reconciliation process during the 'continueReconcilePalette' operation.
    
    :var clientID: A unique clientId passed by the client in order to identify the reconciliation of the palette
    between subsequent calls to 'continueReconcilePalette'.
    :var paletteOrDisclosingObject: The Fnd0ModelViewPalette  or the object to which Fnd0ModelViewPalette  is
    associated using Fnd0DisclosedViewList relation.
    :var mvProxiesOrMvGroups: List of Fnd0ModelViewProxy  or the  Fnd0ModelViewGroup objects that need to be
    reconciled. Can be used to do a partial reconciliation or just the ModelViewProxy or Group of interest.
    :var doEntirePalette: Specify false to indicate whether to reconcile only the Fnd0ModelViewProxy or
    Fnd0ModelViewGroup specified in 'mvProxiesOrMvGroups', or true to continue with reconciliation of the entire
    palette.
    :var configRecipe: The top line (BOMLine) of the disclosing object or a VisStructureContext which has the product
    structure configuration information.
    """
    clientID: str = ''
    paletteOrDisclosingObject: BusinessObject = None
    mvProxiesOrMvGroups: List[BusinessObject] = ()
    doEntirePalette: bool = False
    configRecipe: BusinessObject = None


@dataclass
class ReconcilePaletteResponse(TcBaseObj):
    """
    This structure contains the reconciliation information reponse for each Fnd0ModelViewProxy  on the
    Fnd0ModelViewPalette. The clientID signifies maintains the correlation between multiple calls to the the server. 
    The 'origConfigInfo' and 'reconcileConfigInfo' are returned only if there is a configuration mis-match between the
    configuration associated to Fnd0ModelViewPalette and the one used for reconciliation.
    
    :var palMVPReconcileInfos: A list of 'PaletteModelViewProxyReconcileInfo', one for each Fnd0ModelViewProxy  on the
    Fnd0ModelViewPalette  that has been reconciled.
    :var clientID: A unique clientId passed by the client to correlate multiple subsequent 'continueReconcilePalette'
    operations .
    :var finished: True if the reconciliation is complete.
    :var configurationMisMatch: Returned as true during 'startReconcilePalette' if there is a configuration mismatch
    between the product configuration associated with the Fnd0ModelViewPalette  and the input product configuration.
    Used to warn the user.
    :var origConfigInfo: The configuration information that is associated to the Fnd0ModelViewPalette, presented in a
    form that can be directly displayed to the user.
    :var reconcileConfigInfo: The configuration information for the current structure that is used for reconciliation.
    :var estimatedMVPsLeft: The number of Fnd0ModelViewProxy objects remaining to be reconciled.
    :var percentComplete: The percentage of total Fnd0ModelViewProxy objects on the Fnd0ModelViewPalette where
    reconciliation is complete.
    :var serviceData: 'ServiceData' is used to communicate partial errors to the client.
    """
    palMVPReconcileInfos: List[PaletteModelViewProxyReconcileInfo] = ()
    clientID: str = ''
    finished: bool = False
    configurationMisMatch: bool = False
    origConfigInfo: ConfigurationDisplayInfo = None
    reconcileConfigInfo: ConfigurationDisplayInfo = None
    estimatedMVPsLeft: int = 0
    percentComplete: float = 0.0
    serviceData: ServiceData = None


@dataclass
class CandidateModelViewStructNodeInfo(TcBaseObj):
    """
    The structure contains  BOMLine or  Cpd0DesignElement on which the candidate Fnd0ModelViewProxy was identified. If
    a BOMLine is returned then also the clone stable occ id  chain which identifies BOMLine in a structure is also
    returned.
    
    :var structNode: A BOMLine or Cpd0DesignElement object on which the candidate was identified.  A BOMLine is
    returned only if a BOMLine is sent as 'configRecipe' input.
    :var cloneStableIdChain: List of strings representing a chain of clone stable ids that lead to the 'structNode' in
    the product structure.
    The clone stable occurrence id values are ordered from the top BOMLine incase 'structNode' is a BOMLine with line
    object being ItemRevision. Alternatively, if a the 'csIdContextOccurrence' is a subset in the Workset, and only a
    single cs_id value for the 'structNode' would be needed since a given model element (Mdl0ModelElement) in a subset
    (Cpd0DesignSubsetElement)
    can be identified by such a single clone stable id.
    :var csIdContextOccurrence: Contains either the subset in which the structure node resides (either Cpd0SubsetLine
    or Cpd0DesignSubsetElement.). This object gives context to the associated 'cloneStableIdChain' value(s). In case if
    the 'structNode' is a BOMLine whose line object is ItemRevision then this would be empty since the top line gives
    the context for the 'cloneStableIdChain'.
    """
    structNode: BusinessObject = None
    cloneStableIdChain: List[str] = ()
    csIdContextOccurrence: BusinessObject = None


@dataclass
class ConfigurationDisplayInfo(TcBaseObj):
    """
    This structure contains the configuration information that can be displayed to the user if there is a mis-match
    between the configuration found on the Fnd0ModelViewPalette and the configuration used for palette reconciliation.
    Note: The information returned by this structure should be used only to display configuration information.
    
    :var configWindow: The configuration window (BOMWindow) that represents the configuration.In case when the previous
    configuration information is returned during reconciliation process the BOMWindow is valid until the reconciliation
    is complete.
    :var revisionRule: The RevisionRule used for the configuration.
    :var revRuleConfigEntriesAsStrings: A list of strings describing the configuration entries in the RevisionRule.
    :var configOptions: A list of options that is being used during configuration such as "hide unconfigured lines" etc.
    :var variantRules: A list of VariantRule objects used for the configuration.
    """
    configWindow: BusinessObject = None
    revisionRule: BusinessObject = None
    revRuleConfigEntriesAsStrings: List[str] = ()
    configOptions: List[ConfigurationOptionDisplayInfo] = ()
    variantRules: List[BusinessObject] = ()


@dataclass
class ConfigurationOptionDisplayInfo(TcBaseObj):
    """
    A structure to represent the configuration option and its values. Depending upon the type of option value the
    appropriate values member would be populated. Say for example for the option "Show Unconfigured Variants" the
    'boolValues' would be populated with a single value and the rest of the values member would be empty. The caller
    can determine the type of option value by checking the value memberthat has been populated.
    
    :var optionName: The name of the option use for configuration.
    :var boolValues: A list of boolean values if the option is a boolean.
    :var doubleValues: A list of double values if the option is a double.
    :var intValues: A list of integer values if the option is an integer.
    :var strValues: A list of char or string values if the option is a character or string.
    :var dateValues: A list of date values if the option is a date.
    :var refValues: A list of objects values if the option is a object.
    """
    optionName: str = ''
    boolValues: List[bool] = ()
    doubleValues: List[float] = ()
    intValues: List[int] = ()
    strValues: List[str] = ()
    dateValues: List[datetime] = ()
    refValues: List[BusinessObject] = ()


"""
Generic map of boolean names to values (string, bool). Commonly used to contain selected options from a client.
"""
BoolMap2 = Dict[str, bool]



import enum


class Severity(enum.Enum):
    information = 1
    warning = 2
    error = 3
    user_error = 4


class TcError(Exception):
    def __init__(self, message: str, code: int, level: Severity):
        super(TcError, self).__init__(f"[{level.name}] Code {code} - {message}")
        self.message = message
        self.code = code
        self.level = level


class ServiceException(TcError):
    pass


class InvalidCredentialsException(TcError):
    pass


class InternalServerException(TcError):
    pass



from __future__ import annotations

from tcsoa.gen.Core._2015_07.DataManagement import CreateIn2, CreatableSubBONamesInput, LocalizedPropertyValuesResponse, PropertyNamingruleInfo, DomainOfObjectOrTypeResponse, GetDomainInput, DeepCopyDataInput, CreatableSubBONamesResponse, GetDeepCopyDataResponse
from tcsoa.gen.Core._2008_06.DataManagement import CreateResponse
from tcsoa.gen.Core._2013_05.DataManagement import GenerateNextValuesResponse
from typing import List
from tcsoa.gen.Core._2010_04.DataManagement import PropertyInfo
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def createRelateAndSubmitObjects2(cls, createInputs: List[CreateIn2]) -> CreateResponse:
        """
        This is a generic operation for creation of business objects. This will also create any secondary (compounded)
        objects that need to be created, assuming the CreateInput2 for the secondary object is represented in the
        recursive CreateInput2 object e.g. Item is primary object that also creates Item Master and ItemRevision.
        ItemRevision in turn creates ItemRevision Master. The input for all these levels is passed in through the
        recursive CreateInput2 object.
        This operation also performs following tasks:
        &bull;  Relate the created business object and the additional data passed in through the dataToBeRelated input
        of CreateIn2 object.
        &bull;  Submit the created business object to the workflow process. The input for creating the workflow process
        is passed in through the workflowData input of CreateIn2 object.
        &bull;  Relate the created business object to the input target object.
        
        Use cases:
        Use this operation to create an object after obtaining user input on the fields of the create dialog. This call
        is typically preceded by a call to Teamcenter::Soa::Core::_2008_06::PropDescriptor::getCreateDesc or to the
        Client Meta Model layer to retrieve Create Descriptor for a business object.
        Create Item
        For example, to create an Item, client will get the Create Descriptor associated with the Item from the client
        Meta model (The associated descriptor type can be found by looking at the constant value for the CreateInput
        constant that is attached to Item). Alternatively, for clients that do not use the client Meta model, the
        Descriptor for Item can be obtained by invoking getCreateDesc operation. The descriptor information can then be
        used to populate the Create dialog for the business object. Once the Create dialog is populated the
        createObjects operation can be called to create the object.
        Create Problem Report
        User want to create a new Problem Report (Change) object and attach an existing Word doc dataset as "Reference"
        or "ProblemItem" relation. Here User also wants to submit the created Problem Report object to the workflow
        with predefined Workflow template for Problem Report object type.
        """
        return cls.execute_soa_method(
            method_name='createRelateAndSubmitObjects2',
            library='Core',
            service_date='2015_07',
            service_name='DataManagement',
            params={'createInputs': createInputs},
            response_cls=CreateResponse,
        )

    @classmethod
    def generateNextValuesForProperties(cls, propertyNamingRuleInfo: List[PropertyNamingruleInfo]) -> GenerateNextValuesResponse:
        """
        This operation generates values for the given properties of an object(s) during create/revise/save as action
        based on the user exits or naming rules configured on those properties.Customer user exits are given priority
        over the naming rules if both of them are configured on the same property. The counter has to be set active on
        the naming rule in order to generate the next value for a property. This operation also performs naming rule
        and multi field key validation on the generated values and return appropriate partial errors if the validation
        fails.
        
        For user exit support, an existing user exit will be called to generate values. Right now we support below
        given user exits for corrosponding Objest type.
        
        Object: Item
        User exit name: USER_new_item_id
        Property: item_id
        
        Object: ItemRevision
        User exit name: USER_new_revision_id, USER_new_revision_id_from_alt_rule(Baseline)
        Property: item_revision_id
        
        Object: Dataset
        User exit name: USER_new_dataset_id
        Property: pubr_object_id
        
        Object: Dataset
        User exit name: USER_new_dataset_rev
        Property: rev
        
        Object: Identifier
        User exit name: IDFR_new_alt_id, IDFR_new_rev_id(In Revise case)
        Property: idfr_id
        
        Object: CPD Objects
        User exit name: USER_new_cpd_id
        Property: CPD Objects related property
        
        These each user exits need some specific inputs which are required by them to generate ids. These inputs are
        part of "generateNextValuesIn" structure and are described in details in its description.
        
        
        Use cases:
        a)    User clicks on assign button in Create/Revise/Saveas dialog:
        
        Autoassignable properties are those that have either a user exit or a naming rule with counter configured.A
        constant "autoassignable" is defined on the PropertyDescription class and its value can be obtained using
        PropertyDescription.getConstant() API. "Assign" button is displayed in create/revise/save as dialog to populate
        their values.
        
        This operation is used to generate the values for the autoassignable properties when the user clicks on the
        "Assign" button.The caller should collect the list of all autoassignable properties that do not have any user
        input and pass them to this operation. If the autoassignable property has a naming rule , the naming rule
        pattern selected by the user for that property should also be passed as input to this operation. In all other
        cases the naming rule pattern should be passed as empty string.
        
        b)    User clicks on finish button in Create/Revise/Saveas dialog:
        
        This operation is also used to generate the values for the autoassignable properties when the user clicks
        "Finish" button in in create/revise/save as dialog. The caller should collect the list of all autoassignable
        properties that do not have any value generated and pass them to this operation. If the autoassignable property
        has a naming rule , the naming rule pattern selected by the user for that property should also be passed as
        input to this operation. In all other cases the naming rule pattern should be passed as empty string.
        """
        return cls.execute_soa_method(
            method_name='generateNextValuesForProperties',
            library='Core',
            service_date='2015_07',
            service_name='DataManagement',
            params={'propertyNamingRuleInfo': propertyNamingRuleInfo},
            response_cls=GenerateNextValuesResponse,
        )

    @classmethod
    def getCreatbleSubBuisnessObjectNames(cls, input: List[CreatableSubBONamesInput]) -> CreatableSubBONamesResponse:
        """
        This operation returns sub business object  names that are displayable to the login user in the object creation
        dialog and their display names for each primary business object given as the input, based on specified context.
        Returned business object lists have exclusions of business objects and their secondary business objects as per
        the exclusion preference and/or exlusion business object names specified in the input. If the context is
        specified as legacy, the sub business objects of the primary business object are returned only if the primary
        business object is listed in the site preference TYPE_DISPLAY_RULES_list_types_of_subclasses. If the context is
        left blank, then all creatable sub business objects are returned. This operation returns the hierarchy of
        creatable objects for each business object it returns.
        
        Use cases:
        Use Case 1: Get all Creatable sub business object names for a given business object
        While creating an object of a business object, user needs to know all the sub business objects that can be
        created. To get all creatable sub business objects for a given business object for the logged in user, this
        operation should be invoked by providing empty value for context. Any specific sub business objects that need
        to be excluded from the returned list, can be specified through exclusionPreference and/or exclusionBONames
        parameters.
        
        Use Case 2: Get Creatable sub business objects for a given primary business object, excluding sub business
        objects from its sub classes
        While creating an object of a primary business object, user needs to know only the sub business objects that
        can be created for the primary business object, excluding the business objects from its sub classes, for
        example, in the legacy Create wizards from Teamcenter Rich Application Client. To get only the creatable direct
        sub business objects for a given primary business object for the logged in user, this operation should be
        invoked by providing legacy as the value for context parameter.
        """
        return cls.execute_soa_method(
            method_name='getCreatbleSubBuisnessObjectNames',
            library='Core',
            service_date='2015_07',
            service_name='DataManagement',
            params={'input': input},
            response_cls=CreatableSubBONamesResponse,
        )

    @classmethod
    def getDeepCopyData(cls, deepCopyDataInput: DeepCopyDataInput) -> GetDeepCopyDataResponse:
        """
        This operation returns information required to perform save-as/revise after user changes the default copy
        action to one of the following copy actions for one of the secondary objects:
        - Copy As Object
        - Revise Object
        - Revise and Relate to Latest
        
        
        
        Use cases:
        After a user changes the copy action for one of the secondary objects in the DeepCopy panels, Client will need
        to call this operation to construct the DeepCopy panels in save-as wizard for user input for that secondary
        object. Once the user input is received, client can make subsequent invocation to the
        DataManagement.saveAsObjectsAndRelate operation to execute SaveAs on the object. 
        """
        return cls.execute_soa_method(
            method_name='getDeepCopyData',
            library='Core',
            service_date='2015_07',
            service_name='DataManagement',
            params={'deepCopyDataInput': deepCopyDataInput},
            response_cls=GetDeepCopyDataResponse,
        )

    @classmethod
    def getDomainOfObjectOrType(cls, inputs: List[GetDomainInput]) -> DomainOfObjectOrTypeResponse:
        """
        This operation identifies the application domain information of the input design artifact object or type name
        and the domain value will be returned as part of the response. The input object can be any WorkspaceObject. The
        application domain where the object or the type belongs to is returned as domain value.  Out of the box the
        supported application domains are "Mechanical" and "Automation". The domain information is identified based on
        preferences. Every application domain will have a preference with type name applicable for it and preference
        with syntax as "MECH_domain_types_"+DomainName  [eg. MECH_domain_types_Automation]. And the domain name will be
        added to a master preference "MECH_domain_list".  For legacy objects whose domain information cannot be
        identified from type,  will have to have Fnd0AppDomain object associated with it. And Fnd0AppDomain will have
        the domain information. Based on the Fnd0AppDomain or the preference the domain of the input object or type
        will be identified.
        
        Use cases:
        When domain information for a design artifact (WorkspaceObject) cannot be identified by type name, user uses
        the exposed utility associate_domain_data to associate domain information.
        User, who wants to know the application domain where a design artifact (WorkspaceObject) or type belongs to,
        uses this operation to query for the domain information.
        """
        return cls.execute_soa_method(
            method_name='getDomainOfObjectOrType',
            library='Core',
            service_date='2015_07',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=DomainOfObjectOrTypeResponse,
        )

    @classmethod
    def getLocalizedProperties2(cls, propertyInfo: List[PropertyInfo]) -> LocalizedPropertyValuesResponse:
        """
        Typically business object property values are returned in the locale of the current session, this operation
        returns desired property values in any of the supported locales of the Teamcenter server. String type
        properties may be localized with values for each supported locale, this operation will return the translated
        values for one or more desired locales.
        
        Use cases:
        Retrieve the localized values for localizable property
        
        When running Teamcenter in language environment other than the English, user wants to see the localized
        property value to be displayed in corresponding language in the UI. This operation can be used to fulfill this
        requirement. By providing the desired business object, internal name of the properties, and specific locale
        name(s), this operation will return the localized property value(s) in that particular locale(s) and the
        internal value(s) of the status corresponding to localized value(s) in that locale(s).
        """
        return cls.execute_soa_method(
            method_name='getLocalizedProperties2',
            library='Core',
            service_date='2015_07',
            service_name='DataManagement',
            params={'propertyInfo': propertyInfo},
            response_cls=LocalizedPropertyValuesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ACLInfos(TcBaseObj):
    """
    This structure holds the business object and corresponding effective ACL information derived through access rule
    evaluation.  Each effective ACL contains a list of ACE objects. Each ACE object is a single row in the effective
    ACL table that contains information like granted privileges, revoked privileges, Accessor Type name, Accessor Id
    and the ACL name.
    
    :var workSpaceObject: The business object for which effective ACL was derived.
    :var aclInfos: List of ACE objects returned from the server that make up the effective ACL for the business object.
    """
    workSpaceObject: BusinessObject = None
    aclInfos: List[ACLInfo] = ()


@dataclass
class ExtraProtectionInfo(TcBaseObj):
    """
    This structure contains the derived extra protection information corresponding to a single access privilege on a
    given business object for the given user.
    
    :var privilegeNameInfo: Object that holds the internal name and display name of the access privilege.
    :var verdict: The verdict for the privilege for the user on the object.
    :var rules: A list of access rules evaluated to arrive at the verdict for the privilege.
    :var ruleEvaluation: A list of access rule arrangements involved in arriving at the verdict. Arrangements means the
    order in which the rules are evaluated.
    :var aclNameInfo: Object that holds the internal and display names of the named ACL. For object ACLs the internal
    name is "OBJECT".
    :var accessorTypeNameInfo: Object that holds the internal and display names of the accessor type name corresponding
    to the ACE that involved in arriving at the privilege verdict.
    :var accessorIdInfo: Object that holds the internal and display names of the accessor Id  corresponding to the ACE
    that involved in arriving at the privilege verdict.
    """
    privilegeNameInfo: DisplayNameInfo = None
    verdict: bool = False
    rules: List[str] = ()
    ruleEvaluation: List[str] = ()
    aclNameInfo: DisplayNameInfo = None
    accessorTypeNameInfo: DisplayNameInfo = None
    accessorIdInfo: DisplayNameInfo = None


@dataclass
class ExtraProtectionInfoReport(TcBaseObj):
    """
    This structure holds the business object and corresponding derived extra protection information for all the access
    privileges for the given user.
    
    :var workSpaceObject: POM_object on which extra protection information is derived for the given user.
    :var extraProtectionInfos: The derived extra protection information corresponding to the single business object.
    """
    workSpaceObject: BusinessObject = None
    extraProtectionInfos: List[ExtraProtectionInfo] = ()


@dataclass
class ExtraProtectionInfoResponse(TcBaseObj):
    """
    This structure contains list of extra protection reports one for of each of the given business objects for the
    given user.
    
    :var extraProtectionReports: List of extra protection report objects one for each of the given business objects.
    :var serviceData: Object that holds the partial errors occurred while deriving extra protection  information of any
    of the given business objects.
    """
    extraProtectionReports: List[ExtraProtectionInfoReport] = ()
    serviceData: ServiceData = None


@dataclass
class PrivNamesInfoResponse(TcBaseObj):
    """
    This structure holds the internal and display names of all the Teamcenter access privileges.
    
    :var privNameInfos: A list of DisplayNameInfo objects that holds internal name and display names of each Teamcenter
    privilege objects.
    :var serviceData: Object that holds any of the partial errors that occurred while getting the display names of the
    privileges in Teamcenter. But in fact this operation will not result in any errors as it is returning the names of
    existing privileges. Scope for any errors is very less.
    """
    privNameInfos: List[DisplayNameInfo] = ()
    serviceData: ServiceData = None


@dataclass
class AccessorTypesResponse(TcBaseObj):
    """
    This structure contains the internal names and corresponding display values for all the Accessor Types in
    Teamcenter.
    
    :var accessorTypeNameInfos: List of DisplayNameInfo objects containing the internal name and corresponding display
    name for each of the Accessor Types in Teamcenter.
    :var serviceData: Object that holds any partial errors that occur while getting the display names of the Accessor
    Types in Teamcenter.
    """
    accessorTypeNameInfos: List[DisplayNameInfo] = ()
    serviceData: ServiceData = None


@dataclass
class ACLInfo(TcBaseObj):
    """
    Structure to hold the ACE information for a single row in the effective ACL table like granted privileges, revoked
    privileges, Accessor Type name, Accessor Id and the ACL name. This structure holds both internal names and
    corresponding display values for the names of Accessor Type, Named ACL, Privilege and Accessor ID that constitutes
    a single ACE.
    
    :var grantedPrivsInfo: A list of objects containing the internal and display names of the access privileges that
    are granted to the user on a given business object.
    :var revokedPrivsInfo: A list of  objects containing the internal and display names of the access privileges that
    are revoked to the user on a given business object.
    :var accessorTypeNameInfo: Object containing the internal and display names of the accessor type in the ACE that is
    applicable to the given business object and to the user.
    :var accessorIdInfo: Object containing the internal and display names of the accessor id in the ACE that is
    applicable to the given business object and to the user.
    :var aclNameInfo: Object containing the internal and display names of the ACL in the ACE that is applicable to the
    given business object and to the user.
    """
    grantedPrivsInfo: List[DisplayNameInfo] = ()
    revokedPrivsInfo: List[DisplayNameInfo] = ()
    accessorTypeNameInfo: DisplayNameInfo = None
    accessorIdInfo: DisplayNameInfo = None
    aclNameInfo: DisplayNameInfo = None


@dataclass
class ACLInfoResponse(TcBaseObj):
    """
    This structure holds the list of effective ACL reports for each of the given business objects.  Each effective ACL
    report contains the business object and corresponding effective ACL information.
    
    :var aclInfosList: List of derived Effective ACL reports for each of the given business objects.
    :var serviceData: Object that holds the partial errors that occurred while deriving the effective ACL information
    for any of the given business objects.
    """
    aclInfosList: List[ACLInfos] = ()
    serviceData: ServiceData = None


@dataclass
class DisplayNameInfo(TcBaseObj):
    """
    DisplayNameInfo structure holds the internal name and corresponding display name of objects like privilege, named
    ACL, Accessor type and Accessor Id.
    
    :var internalName: String representing the internal name of an access privilege or a named ACL or an accessor type
    or an accessor.
    :var displayName: String representing the display name of an access privilege or a named ACL or an accessor type or
    an accessor.
    """
    internalName: str = ''
    displayName: str = ''



from __future__ import annotations

from typing import List
from tcsoa.gen.Multisite._2014_10.ImportExportTCXML import RemoteImportResponse, RemoteExportResponse, RemoteImportUIDInfo, RemoteExportInfo, RemoteImportInfo
from tcsoa.base import TcService


class ImportExportTCXMLService(TcService):

    @classmethod
    def remoteExport(cls, rinfos: List[RemoteExportInfo]) -> RemoteExportResponse:
        """
        This operation exports targeted objects from the local site to the target sites using the Multi-Site TCXML
        payload. The target objects are specified in the RemoteExportInfo structure. The options also specified in this
        structure affect the final state of the exported objects at the target sites. The objects must be owned at the
        local site. 
        
        Use cases:
        In the Multi-Site federation user can share the business objects among the participating sites, the client
        shares business object(s) using the remote import or remote export functionality. This operation supports the
        following use case.
        The remote export (i.e. push) of business objects owned at the local site to the target site. The business
        object will exist at the target site either a replica or local object depending on options at the conclusion of
        the operation.
        """
        return cls.execute_soa_method(
            method_name='remoteExport',
            library='Multisite',
            service_date='2014_10',
            service_name='ImportExportTCXML',
            params={'rinfos': rinfos},
            response_cls=RemoteExportResponse,
        )

    @classmethod
    def remoteImport(cls, rinfos: List[RemoteImportInfo]) -> RemoteImportResponse:
        """
        The remoteImport operation imports targeted objects from their source site to the local site using the
        Multi-Site TCXML payload. The inputs include various options which will affect the final state of the Business
        Objects at the local site.
        
        Use cases:
        In the Multi-Site federation user can share the business objects among the participating sites, the client
        shares business object(s) using the remote import or remote export functionality. This operation supports the
        following use case.
        The remote import (i.e. pull) of business objects owned at the target site to the local site. The business
        object will exist at the local site as either a replica or local object depending on options at the conclusion
        of the operation. They will be updated if they already exist in the local site.
        """
        return cls.execute_soa_method(
            method_name='remoteImport',
            library='Multisite',
            service_date='2014_10',
            service_name='ImportExportTCXML',
            params={'rinfos': rinfos},
            response_cls=RemoteImportResponse,
        )

    @classmethod
    def remoteImportByUID(cls, rinfos: List[RemoteImportUIDInfo]) -> RemoteImportResponse:
        """
        The remoteImport operation imports targeted objects from their source site to the local site using the
        Multi-Site TCXML payload. The inputs include various options which will affect the final state of the Business
        Objects at the local site.
        
        Use cases:
        In the Multi-Site federation user can share the business objects among the participating sites, the client
        shares business object(s) using the remote import or remote export functionality. This operation supports the
        following use case.
        The remote import (i.e. pull) of business objects owned at the target site to the local site. The business
        object will exist at the local site as either a replica or local object depending on options at the conclusion
        of the operation. They will be updated if they already exist in the local site.
        """
        return cls.execute_soa_method(
            method_name='remoteImportByUID',
            library='Multisite',
            service_date='2014_10',
            service_name='ImportExportTCXML',
            params={'rinfos': rinfos},
            response_cls=RemoteImportResponse,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List


@dataclass
class ContextObjectResponse(TcBaseObj):
    """
    This structure contains the list of all the objects related to the current object within context and the
    serviceData object.
    
    :var objectMap: A map of input business object to a list of related Item, Processes and ChangeRequestRevision 
    (business object/ vector). The DesignContext application then adds these objects to the respective lists i.e.
    Workparts list, ChangeRequestRevision list or the Processes list based on the type of object. If no related object
    is found empty map is returned.
    :var serviceData: The ServiceData objects containing error codes and error messages in case of failure.
    """
    objectMap: RelatedObjectsInfo = None
    serviceData: ServiceData = None


"""
This map contains a list of related objects for each input object.
Key: The input object is the key.
Values: A list of objects that related to the input object are the values.
"""
RelatedObjectsInfo = Dict[BusinessObject, List[BusinessObject]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, VariantRevision, WorkspaceObject, Variant, BOMWindow
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ModularOption(TcBaseObj):
    """
    Describes a modular option information details returned by operations ''getBOMVariantConfigOptions'' and
    ''getModuleOptionsForBom''.
    
    :var optionId: Current session Option ID given by OVE.
    :var optionName: The option name.
    :var operationTypes: Array of operation types (to, less than, greater than, ect.) in case of Int, Real Options
    :var basedOnOption: Based on option details.  This is available only if option type is not native.
    :var mvlDefinitions: MVL Definition Stored
    :var optionDescription: Option Description
    :var optionValueType: The option value data type: String/Int/Real/Logical
    :var optionType: The type of option: Native/Presents/Implements/External
    :var allowedValues: List of allowed values attached to an option
    :var defaultValue: Option default value
    :var optionScope: The scope of the option: Public / Private
    :var minValues: Array of min values, in case of Int & Real options
    :var maxValues: Array of max values, in case of Int & Real options
    """
    optionId: int = 0
    optionName: str = ''
    operationTypes: List[int] = ()
    basedOnOption: BasedOnOptionInfo = None
    mvlDefinitions: List[str] = ()
    optionDescription: str = ''
    optionValueType: int = 0
    optionType: int = 0
    allowedValues: List[str] = ()
    defaultValue: str = ''
    optionScope: int = 0
    minValues: List[float] = ()
    maxValues: List[float] = ()


@dataclass
class ModularOptions(TcBaseObj):
    """
    List of options and MVL (Modular variant Language) string for current Module or BOMLine.
    
    :var options: List of available module options for a BOMLine.
    :var mvl: MVL attached to Item or ItemRevision (module).
    """
    options: List[ModularOption] = ()
    mvl: str = ''


@dataclass
class ModularOptionsForBomResponse(TcBaseObj):
    """
    The return value of the getModularOptionsForBom SOA.
    
    :var optionsOutput: A vector of ModularOptionsOutput that contains the modular options of the requested BOM lines /
    windows
    :var serviceData: The service data for errors and returned objects.
    """
    optionsOutput: List[ModularOptionsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ModularOptionsInfo(TcBaseObj):
    """
    Wrapper structure to encapsulate an input BOMLine and all its Modular Options information, used by
    ''ModularOptionsOutput'' in response of operation ''getModularOptionsForBom''.
    
    :var bomLine: A BOM line
    :var options: Modular options and MVL attached to a given BOMLine.
    """
    bomLine: BOMLine = None
    options: ModularOptions = None


@dataclass
class ModularOptionsInput(TcBaseObj):
    """
    The input to the getModularOptionsForBom SOA Service call
    
    :var bomWindow: The current BOM window
    :var bomLines: Selected BOM lines
    """
    bomWindow: BOMWindow = None
    bomLines: List[BOMLine] = ()


@dataclass
class ModularOptionsOutput(TcBaseObj):
    """
    The output of the 'getModuleOptionsForBom' SOA call.
    
    :var bomWindow: The BOM Window
    :var optionsInfo: Modular options for BOMLine objects in the given BOMWindow.
    """
    bomWindow: BOMWindow = None
    optionsInfo: List[ModularOptionsInfo] = ()


@dataclass
class BOMVariantConfigOptionResponse(TcBaseObj):
    """
    Repose Object for getBOMVariantConfigOption
    
    :var output: Output Object
    :var serviceData: Service data for errors & returned objects
    """
    output: BOMVariantConfigOutput = None
    serviceData: ServiceData = None


@dataclass
class BOMVariantConfigOutput(TcBaseObj):
    """
    Output Structure of operation ''getBOMVariantConfigOptions'' for a given input BOMWindow.
    
    :var bomWindow: Bom Window
    :var configuredOptions: Current Configured Options
    :var dbSOSOrSVR: Saved SOS or Variant Rule currently applied on BOMWindow.
    """
    bomWindow: BOMWindow = None
    configuredOptions: List[BOMVariantConfigurationOption] = ()
    dbSOSOrSVR: WorkspaceObject = None


@dataclass
class BOMVariantConfigurationOption(TcBaseObj):
    """
    Returned object having information of Option & configuration by operation ''getBOMVariantConfigOptions''. This will
    information about a single variant Option.
    
    :var variantType: Type of variant, if Legacy or Modular
    :var howSet: Option how configured (User Set, Derived, Defaulted, variant Item or Unset )
    :var valueSet: current configured value
    :var whereSet: Used only in case of Classic Option if the option was defaulted or derived, this is returned as the
    Id of the Item that contained the expression that defined this option value. If the option was set by a saved
    variant rule, this is returned as the name of the rule.
    :var defaultSet: Default Configured way (unset, default, derived default)
    :var defaultValue: Default value set
    :var modularOption: Modular Option Object if current object type is Modular
    :var classicOption: Legacy Option if option type is Legacy
    """
    variantType: BOMVariantType = None
    howSet: int = 0
    valueSet: str = ''
    whereSet: str = ''
    defaultSet: int = 0
    defaultValue: str = ''
    modularOption: ModularOption = None
    classicOption: ClassicOption = None


@dataclass
class BasedOnOptionInfo(TcBaseObj):
    """
    "Based On" option information for variants. In case of Modular all options having type Presents, Implemented to
    External are "based on" options.
    
    :var basedOnType: Option Type: Presents / Implements / External.
    :var path: Presented Path, filled when option type is Presented.
    :var basedOptionId: Based option ID.
    :var owningModuleKey: Owning item / module id/key.
    :var owningOptionName: Owning option name.
    """
    basedOnType: int = 0
    path: str = ''
    basedOptionId: int = 0
    owningModuleKey: str = ''
    owningOptionName: str = ''


@dataclass
class ClassicOption(TcBaseObj):
    """
    Legacy Option information details returned by operation ''getBOMVariantConfigOptions''.
    
    :var optionName: Option Name
    :var optionDescription: Option Description
    :var itemId: Item Id on which Option is defined
    :var optionValues: Allowed Values in Option
    :var variant: Legacy Option Variant Object. Created 1 each for each legacy option. Using this SOA consumer can do
    further operations on RAC Side if required.
    :var variantRev: Legacy Option variant ItemRevision Object. Created 1 each for each legacy option. It stores option
    values as enum. Using this SOA consumer can do further operations on RAC Side if required.
    """
    optionName: str = ''
    optionDescription: str = ''
    itemId: str = ''
    optionValues: List[str] = ()
    variant: Variant = None
    variantRev: VariantRevision = None


class BOMVariantType(Enum):
    """
    Type of Variant (legacy Or Modular)
    """
    BOM_LEGACY = 'BOM_LEGACY'
    BOM_MODULAR = 'BOM_MODULAR'



from __future__ import annotations

from tcsoa.gen.UiConfig._2014_11.UiConfig import ColumnConfigData
from tcsoa.gen.BusinessObjects import Fnd0Client, Fnd0AbstractCommand, Fnd0UIConfigCollectionRel, Fnd0CommandCollection, Fnd0ClientScope
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetUIConfigResponse(TcBaseObj):
    """
    DEPRECATED: This structure returns information to the client about column configuration and command applicability.
    The ServiceData contains information about errors encountered during processing.
    
    :var serviceData: ServiceData structure containing errors and command, command collection and icon objects. If
    there is an error retrieving the configuration information, the error added to the ServiceData as a partial error.
    :var uiConfigInfo: List of configuration information including command and column information
    """
    serviceData: ServiceData = None
    uiConfigInfo: List[ClientConfigurations] = ()


@dataclass
class ClientConfigurations(TcBaseObj):
    """
    DEPRECATED: This structure contains command and column configuration data for the indicated client.
    
    :var client: The client for which the configurations are applicable.
    :var columnConfigurations: List of column configurations.
    :var commandConfigurations: List of command configurations.
    """
    client: str = ''
    columnConfigurations: List[ColumnConfigData] = ()
    commandConfigurations: List[CommandConfigData] = ()


@dataclass
class CommandCollectionInfo(TcBaseObj):
    """
    DEPRECATED: Contains a command collection and indexes to its children commands or command collections.
    
    :var childIsCollection: Flag indicating if the child is a command or a command collection.
    :var childIsVisible: Flag indicating if the command or collection is visible.
    :var childConfigIndex: Index of the child in either the CommandConfigData:commands list or the
    CommandConfigData:cmdsCollections list.
    :var commandCollection: The Teamcenter command collection object.
    """
    childIsCollection: List[bool] = ()
    childIsVisible: List[bool] = ()
    childConfigIndex: List[int] = ()
    commandCollection: Fnd0CommandCollection = None


@dataclass
class CommandConfigData(TcBaseObj):
    """
    DEPRECATED: This structure returns information about the command configuration definitions for a client scope URI.
    
    :var scopeName: The scope for which the command data is applicable.
    :var hostingClient: The hosting client for which the command data is applicable.
    :var clientScope: The client scope URI containing the list of command configurations.
    :var cmdCollections: List of all top level and child command collections in the client scope.
    :var cmdCollectionIndex: Index into cmdCollections list for top level command collections in the client scope.
    :var commands: List of all command children accessible in the client scope.
    :var commandCollectionRels: List of all commandCollectionRel objects that associate top level command collections
    with client scope.
    """
    scopeName: str = ''
    hostingClient: Fnd0Client = None
    clientScope: Fnd0ClientScope = None
    cmdCollections: List[CommandCollectionInfo] = ()
    cmdCollectionIndex: List[int] = ()
    commands: List[Fnd0AbstractCommand] = ()
    commandCollectionRels: List[Fnd0UIConfigCollectionRel] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Fnd0DiagramRevision, ImanRelation, TransferMode, Fnd0DiagramTmplRevision, Fnd0ShapeRelation
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetDiagramMembersInputInfo(TcBaseObj):
    """
    'GetDiagramMembersInputInfo' structure represents the parameters required to get specific types of members from the
    diagram.
    
    :var membershipType: The membership type.
    :var diagramRevision: The Diagram Revision object.
    :var selectedObject: The Diagram Root object.
    """
    membershipType: int = 0
    diagramRevision: Fnd0DiagramRevision = None
    selectedObject: BusinessObject = None


@dataclass
class GetDiagramMembersOutput(TcBaseObj):
    """
    'GetDiagramMembersOutput' structure represents the output parameters.
    
    :var diagramMembers: A list of structure which holds the primary object, the shape relation object, the persistent
    object, the type of the object and a Boolean to indicate if user has manually removed this object's shape from the
    diagram.
    :var relationsOnDiagram: A list of structure which holds the relation object, the shape relation object, the
    primary and secondary objects of the relation, the type of the relation object and a Boolean to indicate if user
    has manually removed this object's shape from the diagram.
    """
    diagramMembers: List[DiagramMembers] = ()
    relationsOnDiagram: List[RelationsOnDiagram] = ()


@dataclass
class GetDiagramMembersResponse(TcBaseObj):
    """
    'GetDiagramMembersResponse' structure represents the output of the get diagram members operation.
    
    :var outputs: A list of 'GetDiagramMembersOutput' structures.
    :var serviceData: ServiceData
    """
    outputs: List[GetDiagramMembersOutput] = ()
    serviceData: ServiceData = None


@dataclass
class OpenDiagramInputInfo(TcBaseObj):
    """
    'OpenDiagramInputInfo' structure represents the parameters required to open a diagram.
    
    :var selectedObject: The Teamcenter object for which a Visio diagram will be created, either a Workspace object or
    a BOM Line.
    :var diagramRevision: The Diagram revision object.
    """
    selectedObject: BusinessObject = None
    diagramRevision: Fnd0DiagramRevision = None


@dataclass
class OpenDiagramOutput(TcBaseObj):
    """
    OpenDiagramOutput structure represents the parameters as a result of opening a diagram.
    
    :var diagramTmplFileTickets: A list of FMS tickets to the diagram stencils files associated with the diagram
    template.
    :var diagMappingFileTicket: FMS ticket to the diagram's property map file associated with the diagram template.
    :var diagramFileTicket: FMS ticket to the diagram's vdx file which is stored in the transient volume.
    :var diagramMembers: A list of structure which holds the primary object, the shape relation object, the persistent
    object, the type of the object and a Boolean to indicate if user has manually removed this object's shape from the
    diagram.
    :var relationsOnDiagram: A list of structure which holds the relation object, the shape relation object, the
    primary and secondary objects of the relation, the type of the relation object and a Boolean to indicate if user
    has manually removed this object's shape from the diagram.
    :var objectUIDvsShapeID: The map of the UIDs of all the objects and the Visio unique ids of their corresponding
    shapes (string/string).
    :var appDomain: The Application Domain name of the diagram.
    :var startObject: The root object of the diagram.
    """
    diagramTmplFileTickets: List[str] = ()
    diagMappingFileTicket: str = ''
    diagramFileTicket: str = ''
    diagramMembers: List[DiagramMembers] = ()
    relationsOnDiagram: List[RelationsOnDiagram] = ()
    objectUIDvsShapeID: ObjectUIDVsShapeID = None
    appDomain: str = ''
    startObject: BusinessObject = None


@dataclass
class OpenDiagramResponse(TcBaseObj):
    """
    'OpenDiagramResponse' structure represents the output of the open diagram operation.
    
    :var openDiagOutput: A list of OpenDiagramOutput structures.
    :var serviceData: The Service Data.
    """
    openDiagOutput: List[OpenDiagramOutput] = ()
    serviceData: ServiceData = None


@dataclass
class RelationsOnDiagram(TcBaseObj):
    """
    'RelationsOnDiagram' structure holds the details of the relations shown on the diagram.
    
    :var relation: The relation object to be shown on the diagram.
    :var shapeRelation: The relation object, which holds the shape information of Visio shape.
    :var primaryObject: The primary object of the relation, required to show the relation between the members.
    :var secondaryObject: The secondary object of the relation, required to show the relation between the members.
    :var relationType: The TC type name of the relation object.
    :var isRelationMemberOmitted: If true, indicates that the relation shape was removed from the diagram by the user.
    Such shapes can be restored.
    """
    relation: ImanRelation = None
    shapeRelation: Fnd0ShapeRelation = None
    primaryObject: BusinessObject = None
    secondaryObject: BusinessObject = None
    relationType: str = ''
    isRelationMemberOmitted: bool = False


@dataclass
class SaveDiagramInputInfo(TcBaseObj):
    """
    'SaveDiagramInputInfo' structure represents the parameters required to save a diagram.
    
    :var diagFileTicket: FMS ticket to the diagram's .vdx file to be saved.
    :var diagImageFileTicket: FMS ticket to the diagram's preview image to be saved.
    :var diagramRevision: The diagram revision object.
    :var selectedObject: The Teamcenter business object.
    """
    diagFileTicket: str = ''
    diagImageFileTicket: str = ''
    diagramRevision: Fnd0DiagramRevision = None
    selectedObject: BusinessObject = None


@dataclass
class SaveDiagramOutput(TcBaseObj):
    """
    'SaveDiagramOutput' structure represents the output parameters as a result of saving a diagram.
    
    :var resultObjects: A list of Teamcenter objects, which appear on the diagram, for which shape relations are
    created during the saving of the diagram.
    """
    resultObjects: List[BusinessObject] = ()


@dataclass
class SaveDiagramResponse(TcBaseObj):
    """
    'SaveDiagramResponse' structure represents the output of the save diagram operation.
    
    :var saveDiagOutput: A list of SaveDiagramOutput structures.
    :var serviceData: The Service Data.
    """
    saveDiagOutput: List[SaveDiagramOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateDiagramInputInfo(TcBaseObj):
    """
    The parameters required to create a diagram.
    
    :var selectedObject: Teamcenter business object for which a Visio diagram will be created.
    :var openDiagram: If true, the created Visio diagram will be opened upon creation.
    :var propNamevsValueMap: A map of property names and values (string/string). Valid keys are Description, Name,
    templateName and ID.
    """
    selectedObject: BusinessObject = None
    openDiagram: bool = False
    propNamevsValueMap: PropNamevsPropValueMap = None


@dataclass
class CreateDiagramOutput(TcBaseObj):
    """
    'CreateDiagramOutput' structure represents the output parameters as a result of creating a diagram.
    
    :var diagramTmplFileTickets: A list of FMS tickets to the diagram stencils files associated with the diagram
    template.
    :var diagMappingFileTicket: The FMS ticket to the diagram's property map file associated with the diagram template.
    :var diagramRev: The newly created Diagram Revision object.
    :var diagramMembers: A list of diagram members, each element holds the primary object, the shape relation object,
    the persistent object, the type of the object and a Boolean to indicate if user has manually removed this object's
    shape from the diagram.
    :var relationsOnDiagram: A list of relations on the created diagram. Each element holds the relation object, the
    shape relation object, the primary and secondary objects of the relation, the type of the relation object and a
    Boolean to indicate if user has manually removed this object's shape from the diagram.
    :var appDomain: The Application Domain name of the diagram.
    """
    diagramTmplFileTickets: List[str] = ()
    diagMappingFileTicket: str = ''
    diagramRev: Fnd0DiagramRevision = None
    diagramMembers: List[DiagramMembers] = ()
    relationsOnDiagram: List[RelationsOnDiagram] = ()
    appDomain: str = ''


@dataclass
class CreateDiagramResponse(TcBaseObj):
    """
    'CreateDiagramResponse' structure represents the output of create diagram operation.
    
    :var createDiagOutput: A list of created diagrams
    :var serviceData: The Service Data.
    """
    createDiagOutput: List[CreateDiagramOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateTemplateInputInfo(TcBaseObj):
    """
    CreateOrUpdateTemplateInputInfo structure represents the parameters required to create a diagram template.
    
    :var available: If true, the Diagram Template is available for use.
    :var tmplStencilFileTickets: A list of FMS tickets to the Diagram tool specific stencils or Template files.
    :var tmplMappingFileTicket: FMS ticket to the Property Map xml file.
    :var membershipRule: The Transfer mode, which will be used for traversing the structure for the diagram root object.
    :var relationRule: The Relation Rule which is the list of relations between the objects shown on the diagram.
    :var diagramTmplRev: The updated template object.
    :var propNamevsPropValueMap: A map of property names and values (string/string). Valid keys are Description, Name,
    templateName and ID.
    """
    available: bool = False
    tmplStencilFileTickets: List[str] = ()
    tmplMappingFileTicket: str = ''
    membershipRule: TransferMode = None
    relationRule: List[str] = ()
    diagramTmplRev: Fnd0DiagramTmplRevision = None
    propNamevsPropValueMap: PropNamevsPropValueMap = None


@dataclass
class CreateOrUpdateTemplateOuput(TcBaseObj):
    """
    'CreateOrUpdateTemplateOuput' structure represents the output parameters required for creating or updating a
    diagram template.
    
    :var diagramTmplRev: The created or updated  template object.
    """
    diagramTmplRev: Fnd0DiagramTmplRevision = None


@dataclass
class CreateOrUpdateTemplateResponse(TcBaseObj):
    """
    'CreateOrUpdateTemplateResponse' structure represents the list of 'CreateOrUpdateTemplateOuput' structures as a
    result of creating or updating a diagram template.
    
    :var outputs: A list of CreateOrUpdateTemplateOuput structures, which hold the information of created templates.
    :var serviceData: If true, the created Visio diagram will be opened on creation.
    """
    outputs: List[CreateOrUpdateTemplateOuput] = ()
    serviceData: ServiceData = None


@dataclass
class DiagramMembers(TcBaseObj):
    """
    'DiagramMembers' structure holds the details of the members shown on the diagram.
    
    :var primaryObject: The primary object of the shape shown on the diagram.
    :var shapeRelation: The relation object, which holds the shape information of Visio shape.
    :var persistentObject: The persistent object of the primary object shown on the diagram.
    :var objectTCTypeName: The TC type name of the object.
    :var isMemberOmitted: If true, indicates that the shape was removed from the diagram by the user. Such shapes can
    be restored.
    """
    primaryObject: BusinessObject = None
    shapeRelation: Fnd0ShapeRelation = None
    persistentObject: BusinessObject = None
    objectTCTypeName: str = ''
    isMemberOmitted: bool = False


"""
A Map that holds the collection of object UID for the object that appears on diagram as key vs the ID of the Shape that this object represent on diagram as value.
"""
ObjectUIDVsShapeID = Dict[str, int]


"""
This  map contains the property name and its value.
"""
PropNamevsPropValueMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import IRDC
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class IRDCResponse(TcBaseObj):
    """
    The IRDC response structure
    
    :var outIRDC: The list of references to class IRDC.
    :var svcData: The Service Data. Partial errors and failures are updated and returned through this object.
    """
    outIRDC: List[IRDC] = ()
    svcData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.XlsBom._2020_12.Import import ImportExcelData, StructureInfoResp, ImportExcelResp, ExcelImportInput
from tcsoa.base import TcService


class ImportService(TcService):

    @classmethod
    def importExcelAndUpdateMappingGrp(cls, importExcelData: ImportExcelData) -> ImportExcelResp:
        """
        Description:
        Imports an Excel file into Teamcenter and creates the product structure and sets the properties on objects
        based on the values given in Excel file. The input Excel file contains a level column to indicate the level of
        the object in the structure. "Level" and "Object Type" columns are mandatory in the input Excel sheet. The
        input Excel file is parsed to import the data to create product structure in Teamcenter. This operation
        supports the creation of new objects and update of the existing objects. The column headers in excel are mapped
        to their respective properties which are saved as a mapping group on "import". This operation support create
        and update of mapping groups.
        
        Parameters:
        importExcelData2      A list of type and property related data and the file ticket of the Excel file to
        imported.
        
        Use cases:
        Use Case:
        1.    The user creates a new mapping group by mapping the property names with the column headers in the Excel
        sheet, giving a new group name and clicking on "Import" button.
        2.    The user uses an existing mapping group by selecting a group name which populates the existing mappings
        and clicking on "Import" button.
        3.    The user updates an existing mapping group by selecting a group name, changing some of the existing
        mappings and clicking on "Import" button.
        """
        return cls.execute_soa_method(
            method_name='importExcelAndUpdateMappingGrp',
            library='Internal-XlsBom',
            service_date='2020_12',
            service_name='Import',
            params={'importExcelData': importExcelData},
            response_cls=ImportExcelResp,
        )

    @classmethod
    def importExcelAndUpdateMappingGrpAsync(cls, importExcelData: ImportExcelData) -> None:
        """
        Imports an Excel file into Teamcenter and creates the product structure and sets the properties on objects
        based on the values given in Excel file. The input Excel file contains a level column to indicate the level of
        the object in the structure." Level" and "Object Type" columns are mandatory in the input Excel sheet. The
        input Excel file is parsed to import the data to create product structure in Teamcenter. This operation
        supports the creation of new objects and update of the existing objects. The column headers in excel are mapped
        to their respective properties which are saved as a mapping group on "import". This operation support create
        and update of mapping groups.
        
        Use cases:
        1.    The user creates a new mapping group by mapping the property name with the column headers in the Excel
        sheet, giving a new group name and clicking on "Import" button.
        2.    The user uses an existing mapping group by selecting a group name which populates the existing mappings
        and clicking on "Import" button.
        3.    The user updates an existing mapping group by selecting a group name, changing some of the existing
        mappings and clicking on "Import" button.
        """
        return cls.execute_soa_method(
            method_name='importExcelAndUpdateMappingGrpAsync',
            library='Internal-XlsBom',
            service_date='2020_12',
            service_name='Import',
            params={'importExcelData': importExcelData},
            response_cls=None,
        )

    @classmethod
    def getStructureInfoFromExcel(cls, inputData: ExcelImportInput) -> StructureInfoResp:
        """
        This operation retrieves the page of structure information from the input Excel file. The output also contains
        a cursor that defines the place to start the next page location. The cursor must be passed back in to any
        subsequent call to get the next page of nodes.
        """
        return cls.execute_soa_method(
            method_name='getStructureInfoFromExcel',
            library='Internal-XlsBom',
            service_date='2020_12',
            service_name='Import',
            params={'inputData': inputData},
            response_cls=StructureInfoResp,
        )



from tcsoa.gen.ClassificationCommon._2020_01.services import ClassificationService as imp0
from tcsoa.gen.ClassificationCommon._2020_12.services import ClassificationService as imp1
from tcsoa.base import TcService


class ClassificationService(TcService):
    deleteClassificationObjects = imp0.deleteClassificationObjects
    getClassificationObjects = imp0.getClassificationObjects
    importClassificationDefinitions = imp1.importClassificationDefinitions
    saveClassificationObjects = imp0.saveClassificationObjects
    searchClassificationDefinitions = imp1.searchClassificationDefinitions



from __future__ import annotations

from tcsoa.gen.ProjectManagement._2007_01.ScheduleManagement import CreateBaselineContainer, TaskDeliverableData, CreateScheduleContainer, ScheduleObjDeleteContainer, ScheduleDeliverableData, MembershipData, ScheduleCopyContainer
from typing import List
from tcsoa.gen.BusinessObjects import ScheduleTask, Schedule
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ScheduleManagementService(TcService):

    @classmethod
    def copySchedules(cls, scheduleCopyContainer: List[ScheduleCopyContainer]) -> ServiceData:
        """
        Makes a deep copy of the schedule with options to reset work and copy existing baselines.
        """
        return cls.execute_soa_method(
            method_name='copySchedules',
            library='ProjectManagement',
            service_date='2007_01',
            service_name='ScheduleManagement',
            params={'scheduleCopyContainer': scheduleCopyContainer},
            response_cls=ServiceData,
        )

    @classmethod
    def addMemberships(cls, membershipData: List[MembershipData]) -> ServiceData:
        """
        Add resources to the schedule with given membership levels.
        """
        return cls.execute_soa_method(
            method_name='addMemberships',
            library='ProjectManagement',
            service_date='2007_01',
            service_name='ScheduleManagement',
            params={'membershipData': membershipData},
            response_cls=ServiceData,
        )

    @classmethod
    def createNewBaselines(cls, createBaselineContainer: List[CreateBaselineContainer]) -> ServiceData:
        """
        This operation creates a new schedule baselines possibly based on a previous baseline.
        The created objects are returned back in the 'ServiceData' of the response.The information required to create
        new baseline is passed as input parameter to the operation through 'CreateBaselineContainer' data structure.
        """
        return cls.execute_soa_method(
            method_name='createNewBaselines',
            library='ProjectManagement',
            service_date='2007_01',
            service_name='ScheduleManagement',
            params={'createBaselineContainer': createBaselineContainer},
            response_cls=ServiceData,
        )

    @classmethod
    def createSchedule(cls, newSchedules: List[CreateScheduleContainer]) -> ServiceData:
        """
        Create new scheduling object based on the initial
        user's request to the Application Interface.
        """
        return cls.execute_soa_method(
            method_name='createSchedule',
            library='ProjectManagement',
            service_date='2007_01',
            service_name='ScheduleManagement',
            params={'newSchedules': newSchedules},
            response_cls=ServiceData,
        )

    @classmethod
    def createScheduleDeliverableTemplates(cls, scheduleDeliverableData: List[ScheduleDeliverableData]) -> ServiceData:
        """
        Creates new schedule deliverable templates and relates them to the schedule.
        """
        return cls.execute_soa_method(
            method_name='createScheduleDeliverableTemplates',
            library='ProjectManagement',
            service_date='2007_01',
            service_name='ScheduleManagement',
            params={'scheduleDeliverableData': scheduleDeliverableData},
            response_cls=ServiceData,
        )

    @classmethod
    def createTaskDeliverableTemplates(cls, taskDeliverableData: List[TaskDeliverableData]) -> ServiceData:
        """
        Creates new task deliverable templates and relates them to the task.
        """
        return cls.execute_soa_method(
            method_name='createTaskDeliverableTemplates',
            library='ProjectManagement',
            service_date='2007_01',
            service_name='ScheduleManagement',
            params={'taskDeliverableData': taskDeliverableData},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteSchedulingObjects(cls, scheduleObjDeleteContainer: List[ScheduleObjDeleteContainer]) -> ServiceData:
        """
        The operation is used to delete a scheduling object which cannot be deleted by 'deleteObjects' operation. After
        the operation is called, the business logic determines whether to delete the scheduling objects by examining
        the schedule preservation rule, SM_PREVENT_DELETE_STATE. The SM_PREVENT_DELETE_STATE rule is optional and the
        system admin can remove the rule from the system or set its value to an empty list. But if the rule is in the
        system and set to any status or statuses then the value of the rule would be applied.
        The scheduling objects are Schedule, ScheduleTask, TaskDependency and ResourceAssignment.
        """
        return cls.execute_soa_method(
            method_name='deleteSchedulingObjects',
            library='ProjectManagement',
            service_date='2007_01',
            service_name='ScheduleManagement',
            params={'scheduleObjDeleteContainer': scheduleObjDeleteContainer},
            response_cls=ServiceData,
        )

    @classmethod
    def baselineTasks(cls, scheduleTasks: List[ScheduleTask], scheduleBaseline: Schedule) -> ServiceData:
        """
        This operation baselines or re-baselines a task's data in the context of an existing schedule baseline. The
        updated objects or added task baselines are returned back in the service data of the response. This operation
        throws a 'ServiceException' in case of failure. The service exception will contain the error message of the
        failure. Additional errors will be returned in the list of partial errors in the 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='baselineTasks',
            library='ProjectManagement',
            service_date='2007_01',
            service_name='ScheduleManagement',
            params={'scheduleTasks': scheduleTasks, 'scheduleBaseline': scheduleBaseline},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, DatasetType, WorkspaceObject
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExpandGRMRelationsData(TcBaseObj):
    """
    This structure contains information for 'ExpandGRMRelationsData'.
    
    :var otherSideObjects: Objects on the other side of the relationship
    :var relationName: Input GRM relation name
    """
    otherSideObjects: List[BusinessObject] = ()
    relationName: str = ''


@dataclass
class ExpandGRMRelationsOutput(TcBaseObj):
    """
    This structure contains information for Expand GRM Relations Output.
    
    :var inputObject: Object that was expanded
    :var otherSideObjData: List of 'ExpandGRMRelationsData'
    """
    inputObject: BusinessObject = None
    otherSideObjData: List[ExpandGRMRelationsData] = ()


@dataclass
class ExpandGRMRelationsPref(TcBaseObj):
    """
    Expand GRM Relations Pref
    
    :var expItemRev: Flag to expand any Item Revisions that are in the return data
    :var info: List of RelationAndTypesFilter2
    """
    expItemRev: bool = False
    info: List[RelationAndTypesFilter2] = ()


@dataclass
class ExpandGRMRelationsResponse(TcBaseObj):
    """
    Expand GRM Relations Response
    
    :var output: List of SaveQueryCriteriaInfo
    :var serviceData: Standard ServiceData member
    """
    output: List[ExpandGRMRelationsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetAvailableTypesResponse(TcBaseObj):
    """
    The GetAvailableTypesResponse struct resprents return repsonse of all types implemented by the given class.
    
    :var inputClassToTypes: A map of given class names and all according AvailableTypeInfo objects.
    :var serviceData: ServiceData contains any parital error if any failure happens.
    """
    inputClassToTypes: ClassToTypesMap = None
    serviceData: ServiceData = None


@dataclass
class AvailableTypeInfo(TcBaseObj):
    """
    This structure represents string  instance of type and its hierarchy.
    
    :var type: The name of instance type
    :var hierarchies: hierarchies
    """
    type: str = ''
    hierarchies: List[str] = ()


@dataclass
class PurgeSequencesInfo(TcBaseObj):
    """
    The data structure represents all of the data necessary to purge sequences. The structure contains the selected
    object and a flag indicating whether to validate that the selected sequences is the active sequence.
    
    :var validateLatestFlag: A flag indicating if 'inputObject' should be validated that it is the latest sequence of
    the ItemRevision. To purge all previous sequences, set 'validateLatestFlag' true and set 'inputObject' to the
    latest sequence of the ItemRevision.  To purge a specific sequence, set 'validateLatestFlag' false and set
    'inputObject' to the specific sequence of the ItemRevision that should be purged.
    :var inputObject: A business object representing the ItemRevision sequence that should be purged.
    """
    validateLatestFlag: bool = False
    inputObject: WorkspaceObject = None


@dataclass
class ReferenceInfo(TcBaseObj):
    """
    This structure contains information for a given Dataset type.
    
    :var referenceName: Reference name for the input Dataset type
    :var isObject: Flag that signifies the reference is an object if the value is true.  False signifies the reference
    is a file.
    :var fileFormat: The format for reference object.  Valid values are either TEXT, BINARY or OBJECT.
    :var fileExtension: The default extension for a file, such as *.gif or *.doc.
    """
    referenceName: str = ''
    isObject: bool = False
    fileFormat: str = ''
    fileExtension: str = ''


@dataclass
class RelationAndTypesFilter(TcBaseObj):
    """
    This structure contains the relation name to traverse for the given operation and a list of other side object types
    that are valid to return.
    
    :var relationTypeName: Name of relation to traverse
    :var otherSideObjectTypes: List of names of valid other side object types to return
    """
    relationTypeName: str = ''
    otherSideObjectTypes: List[str] = ()


@dataclass
class RelationAndTypesFilter2(TcBaseObj):
    """
    This structure contains the relation name to traverse for the given operation and a list of other side object types
    that are valid to return.
    
    :var relationName: Types of relationships allowed
    :var objectTypeNames: Types of other objects allowed
    """
    relationName: str = ''
    objectTypeNames: List[str] = ()


@dataclass
class BaseClassInput(TcBaseObj):
    """
    The baseClassInput structure represents input data structure to get available Business Objects.
    
    :var baseClass: This is the name of the Business Object for which this operation returns the descendant Business
    Objects.
    :var exclusionTypes: Names of Business Objects (and its secondary Business Objects) to be excluded from returned
    list.
    """
    baseClass: str = ''
    exclusionTypes: List[str] = ()


@dataclass
class SetOrRemoveImmunityInfo(TcBaseObj):
    """
    The data structure represents all of the data necessary to toggle the immunity of an ItemRevision sequence. It
    contains a reference to the ItemRevision sequence whose immunity will be is modified and a flag indicating whether
    immunity will be revoked or applied.
    
    :var setOrRemoveFlag: Flag indicating if immunity should be set or removed on the 'inpoutObject'.  To set immunity,
    the 'setOrRemoveImmunityFlag' should be true.  To remove immunity, the' setOrRemoveImmunityFlag' should be false.
    :var inputObject: ItemRevision sequence object which should be made immune or have immunity removed.
    """
    setOrRemoveFlag: bool = False
    inputObject: WorkspaceObject = None


@dataclass
class ValidateIdsInfo(TcBaseObj):
    """
    The 'ValidateIdsInfo' struct contains the input ids for item and revision and the item type.
    
    :var itemId: Item ID to be validated
    :var revId: Revision ID to be validated
    :var itemType: Item type to validate against, can be null
    """
    itemId: str = ''
    revId: str = ''
    itemType: str = ''


@dataclass
class ValidateIdsOutput(TcBaseObj):
    """
    This structure contains the modified item and revision ids and enum status of the ids respectively.  Valid values
    for the enums are Valid (ids are valid), Invalid (ids are not valid), Modified (ids are not ideal but can be used
    if the user really wants them), Override (ids are not valid, silently replace with modified ones), and Duplicate
    (ids are already allocated in the system).
    
    :var modItemId: Modified item id if specified by Naming Rules
    :var itemIdStatus: Status of the item id represented as a 'ValidateIdsStatus' enum
    :var modRevId: Modified rev id if specified by Naming Rules
    :var revIdStatus: Status of the revision id represented as a 'ValidateIdsStatus' enum
    """
    modItemId: str = ''
    itemIdStatus: ValidateIdsStatus = None
    modRevId: str = ''
    revIdStatus: ValidateIdsStatus = None


@dataclass
class ValidateItemIdsAndRevIdsResponse(TcBaseObj):
    """
    The list of 'ValidateIdsOutput' structures and the 'ServiceData'.
    
    :var output: List of 'ValidateIdsOutput' structures
    :var serviceData: ServiceData contains only error information returned by this operation.
    """
    output: List[ValidateIdsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class WhereReferencedByRelationNameInfo(TcBaseObj):
    """
    The data structure contains the object to perform the where referenced operation on and the filters to narrow the
    list of referencing objects returned.
    
    :var object: Desired business object to find referencing objects of
    :var filter: A list of filters to limit the search of referencing objects
    """
    object: BusinessObject = None
    filter: List[RelationAndTypesFilter] = ()


@dataclass
class WhereReferencedByRelationNameOutput(TcBaseObj):
    """
    This structure contains the object that the where referenced operation was performed on and list of
    'WhereReferencedRelationNameOutputInfo' structures.
    
    :var inputObject: Input object for which referencers were found
    :var info: List of 'WhereReferencedByRelationNameOutputInfo' structures
    """
    inputObject: BusinessObject = None
    info: List[WhereReferencedByRelationNameOutputInfo] = ()


@dataclass
class WhereReferencedByRelationNameOutputInfo(TcBaseObj):
    """
    This structure contains the referencer object, the corresponding relation name, and the level at which the
    referencer was found.
    
    :var referencer: Referencer object of the input object
    :var relationTypeName: Relation type name of how referencer is related to input object
    :var level: Integer value for the level depth where referencer found
    """
    referencer: BusinessObject = None
    relationTypeName: str = ''
    level: int = 0


@dataclass
class WhereReferencedByRelationNameResponse(TcBaseObj):
    """
    The data structure contains the list of the referencing objects that meet the filter criteria for each of the input
    objects.
    
    :var output: A list of filtered referencing objects for each input object
    :var serviceData: Service data
    """
    output: List[WhereReferencedByRelationNameOutput] = ()
    serviceData: ServiceData = None


@dataclass
class DatasetTypeInfo(TcBaseObj):
    """
    This structure contains the Dataset type object reference corresponding to the input Dataset type name and the
    reference information for each valid named reference of the Dataset type.
    
    :var tag: Dataset type reference object for the Dataset type
    :var refInfos: List of valid references for the Dataset type
    """
    tag: DatasetType = None
    refInfos: List[ReferenceInfo] = ()


@dataclass
class DatasetTypeInfoResponse(TcBaseObj):
    """
    The response from 'getDatasetTypeInfo' operation.
    
    :var infos: List of named reference information for each dataset type specified in 'datasetTypeNames' input.
    :var serviceData: The 'ServiceData'.  This operation will populate the 'ServiceData 'plain objects with the dataset
    type object that corresponds to the input dataset type name.
    """
    infos: List[DatasetTypeInfo] = ()
    serviceData: ServiceData = None


class ValidateIdsStatus(Enum):
    """
    A map of enum of the status for validating ids.
    """
    Valid = 'Valid'
    Invalid = 'Invalid'
    Modified = 'Modified'
    Override = 'Override'
    Duplicate = 'Duplicate'


"""
ClassToTypesMap
"""
ClassToTypesMap = Dict[str, List[AvailableTypeInfo]]



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class CheckResult(TcBaseObj):
    """
    Contains the check results for one tool.
    
    :var icoId: The ID of the tool whose parameters were checked.
    :var status: If the status value is 0, checking succeeded. If it is not 0, some check failed. The details why the
    checking failed are described in the report.
    
    :var report: A list of strings that make up the lines of the report.
    """
    icoId: str = ''
    status: int = 0
    report: List[str] = ()


@dataclass
class CheckToolParametersResponse(TcBaseObj):
    """
    A CheckToolParametersResponse structure contains the reports and statuses for all tools that were checked, as well
    as a ServiceData object that may contain error descriptions, if any errors occurred.
    
    :var serviceData: Service data that can contain error descriptions.
    :var checkResults: A list containing a check result for each tool.
    """
    serviceData: ServiceData = None
    checkResults: List[CheckResult] = ()



from __future__ import annotations

from typing import List
from tcsoa.gen.RelationshipViewer._2012_10.NetworkEngine import RGBValue, Type
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GraphTypeListResponse(TcBaseObj):
    """
    Graph type response.
    
    :var views: A list of  role based avaliable graph views. Different role may get different view list.
    :var serviceData: Service data contains the partial errors which used to report any partial failures.
    """
    views: List[GraphViewDescription] = ()
    serviceData: ServiceData = None


@dataclass
class GraphViewDescription(TcBaseObj):
    """
    The description of a graph view
    
    :var name: Name of the graph view. It is also the identifier of graph view.
    :var displayName: Display name of the graph view. It is also the identifier of graph view.
    :var defaultLayout: the default layout of the view. The possible values are: IncrementalHierarchic, Top-to-Bottom,
    Left-to-Right, Bottom-to-Top, Right-to-Left.
    :var defaultExpandDirection: the default expansion direction of the view. The possibile values are: forward,
    backward, all.
    :var visible: Flag to indicate whether the view is visible on graph control panel.
    :var diagramMode: Flag to indicate whether the graph view is in diagram mode by default.
    :var groups: The list of graph view groups.
    """
    name: str = ''
    displayName: str = ''
    defaultLayout: str = ''
    defaultExpandDirection: str = ''
    visible: bool = False
    diagramMode: bool = False
    groups: List[GraphGroup] = ()


@dataclass
class GraphFilter(TcBaseObj):
    """
    Structure for representing information of a filter item, including filter name, RGB values to color the legend of
    the filter at client side. It is used in structure GraphGroup.
    
    :var name: name of the filter
    :var displayName: display name of the filter
    :var color: The color used for legend of this filter at client side
    :var types: The list of types belong to this category.
    """
    name: str = ''
    displayName: str = ''
    color: RGBValue = None
    types: List[Type] = ()


@dataclass
class GraphGroup(TcBaseObj):
    """
    The graph group is a group name with a list of filters. A filter take effect either on node or edge, it filts out
    graph that fall into its type. It is used in structure GraphViewDescription.
    
    :var name: Backend name of the group. Currently, there are two available groups, relations and objects.
    :var filters: The list of graph group filter names. For example: the relations graph group may have Attach,
    Tracability, Structure, Folder, Connectivity as filters. The objects graph group may have Requirement, Functional,
    Logical, Physical, Dataset, Plant as filters.
    """
    name: str = ''
    filters: List[GraphFilter] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, POM_object, Fnd0ChangeContext
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class SetChangeContextResponse2(TcBaseObj):
    """
    The valid input contexts and the corresponding configured objects are added to configuredObjectsMap. If the input
    context is not a valid context for configuring objects, no value is added to configuredObjectsMap for this context,
    and a partial error is added for this context. If the context is valid, and an object is not valid for
    configuration in that context, a partial error is added for that object.
    
    :var serviceData: Contains any partial errors for the operation.
    :var configuredObjectsMap: A map (business object, list of ChangeContextStructOut) between a context and the list
    of configured objects.
    """
    serviceData: ServiceData = None
    configuredObjectsMap: ChangeContextMapOut = None


@dataclass
class ChangeContextStructIn(TcBaseObj):
    """
    Structure containing configuration mode and the list of input objects.
    
    :var changeConfigurationMode: Applicable only when a ChangeNotice  is provided as the input 
    context. Allowed values are "Latest" and "LatestCurrentChangeOnly".
    :var inputObjects: The objects to be configured for the input context.
    """
    changeConfigurationMode: str = ''
    inputObjects: List[POM_object] = ()


@dataclass
class ChangeContextStructOut(TcBaseObj):
    """
    Structure of the context used to configure objects and the list of configured objects.
    
    :var changeContext: A business object that contains the input context and the configuration mode used.
    :var configuredObjs: List of configured objects.
    """
    changeContext: Fnd0ChangeContext = None
    configuredObjs: List[POM_object] = ()


"""
A map between a context and a list of objects to be configured using this context. The context object can be null, in which case the corresponding list of objects will be configured for public. The context can also be a runtime business object which holds the context and the change configuration mode.
"""
ChangeContextMapIn = Dict[BusinessObject, ChangeContextStructIn]


"""
A map between a context and a structure containing the list of objects configured with the context. The context object can be null, in which case the corresponding list of objects will be configured for public context.
"""
ChangeContextMapOut = Dict[BusinessObject, ChangeContextStructOut]



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AddMultiToolCutterResponse(TcBaseObj):
    """
    Output response object for AddMultiToolCutter action.
    
    :var serviceData: Service data that can contain error descriptions.
    
    :var strNewICOUID: UID of newly created ICO.
    :var strNewCutterID: Cutter ID associated with new ICO.
    """
    serviceData: ServiceData = None
    strNewICOUID: str = ''
    strNewCutterID: str = ''


@dataclass
class DeleteMultiToolCutterResponse(TcBaseObj):
    """
    Output response object for DeleteMultiToolCutter action.
    
    :var serviceData: Service data that can contain error descriptions.
    :var lastICOUID: UID of the last ICO. This will be empty when there are multiple ICOs connected with the
    ItemRevision. It will contain the ICO UID when the ItemRevision is connected with a single ICO and all the other
    ICOs were deleted.
    """
    serviceData: ServiceData = None
    lastICOUID: str = ''



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from typing import Dict, List
from tcsoa.gen.BusinessObjects import Item, ItemRevision


@dataclass
class ParentsWhereUsedResponse(TcBaseObj):
    """
    Used to return the response for ParentsWhereUsed operation
    
    :var itemRevisionToParentItemRevisions: Contains the Map of input Item Revisions with the Parent Item Revisions
    where they are used.
    :var totalPageCount: Return the total number of pages created while processing the input Item Revisions.
    :var serviceData: An object of the ServiceData that contains list of added, updated, or deleted objects.  Also
    contains list of any errors which occurred within the call.
    """
    itemRevisionToParentItemRevisions: ItemRevisionToItemRevisionsMap = None
    totalPageCount: int = 0
    serviceData: ServiceData = None


@dataclass
class ProductsWhereUsedRespone(TcBaseObj):
    """
    Used to return the response for ProductsWhereUsed operation.
    
    :var itemRevToProductItems: Contains the Map of input item revisions with the product items they are used in.
    :var totalPageCount: Return the total number of pages created while processing the request. The end user can use
    this count to ask for any page they want back from the operation.
    :var serviceData: An object of the ServiceData that contains list of added, updated, or deleted objects.  Also
    contains list of any errors which occurred within the call.
    """
    itemRevToProductItems: ItemRevToProductItemsMap = None
    totalPageCount: int = 0
    serviceData: ServiceData = None


"""
Defines a map that will be used to return Product Items where the Item Revision are used.
"""
ItemRevToProductItemsMap = Dict[ItemRevision, List[Item]]


"""
Defines a map that will be used to return Parent Item Revisions where the given Item Revisions are used as children.
"""
ItemRevisionToItemRevisionsMap = Dict[ItemRevision, List[ItemRevision]]



from __future__ import annotations

from tcsoa.gen.Ai._2018_06.Ai import GetObjectsByApplicationRefsResponse, ConfigurationInfo
from typing import List
from tcsoa.base import TcService


class AiService(TcService):

    @classmethod
    def getObjectsByApplicationRefs(cls, input: List[ConfigurationInfo]) -> GetObjectsByApplicationRefsResponse:
        """
        This operation finds business objects associated with the list of input ApplicationRef objects. The
        ApplicationRef can be associated with either persisent or runtime business objects. One business object is
        returned for each ApplicationRef. If no associated business object can be found for an ApplicationRef then a
        NULL  is returned for that entry.
        """
        return cls.execute_soa_method(
            method_name='getObjectsByApplicationRefs',
            library='Ai',
            service_date='2018_06',
            service_name='Ai',
            params={'input': input},
            response_cls=GetObjectsByApplicationRefsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ImportInput(TcBaseObj):
    """
    Input for importManufacturingFeatures
    
    :var fileName: The unique name of the PLMXML File in the transient volume.
    :var root: The root object (in the structure to which we import the data). The root is always a BOMLine that
    corresponds to an item in the product structure.
    """
    fileName: str = ''
    root: BusinessObject = None


@dataclass
class ImportResponse(TcBaseObj):
    """
    The output of importManufaturingFeatures
    
    :var serviceData: The service data
    :var logFileFullPath: The full path of the log file (created by the import).
    """
    serviceData: ServiceData = None
    logFileFullPath: str = ''



from __future__ import annotations

from tcsoa.gen.Internal.Administration._2008_06.IRM import ACLInfoResponse
from tcsoa.base import TcService


class IRMService(TcService):

    @classmethod
    def getACLsByType(cls, aclsType: int) -> ACLInfoResponse:
        """
        This operation can be used to get the named Access Control Lists (ACLs) of given type. There are 3 types of
        ACLs in Teamcenter which are of type WORKFLOW, RULETREE and RULEPROJ ACLs. RULETREE ACLs are mainly used Access
        Manager Application, WORLFLOW ACLs are used in Workflow designer application and RULEPROJ ACLs are used in
        Project Administration Application. Following are the integer values corresponding to these types.
        
        0-RULETREE ACLs
        1-WORKFLOW ACLs
        2-RULEPROJ ACLs
        3-ACLs of all types will be fetched
        
        Sending an incorrect integer value as parameter to this operation will not cause any error but the operation
        will not return anything.
        
        
        Use cases:
        This operation can be used in general wherever the ACL names need to be displayed. At present following use
        cases in Teamcenter Rich Application Client (RAC) calls this operation.
        - Display the ACL names in Extra Protection information dialog.
        - Display the ACL names in effective ACL dialog.
        - Display the ACL names in Named ACL panel.
        
        """
        return cls.execute_soa_method(
            method_name='getACLsByType',
            library='Internal-Administration',
            service_date='2008_06',
            service_name='IRM',
            params={'aclsType': aclsType},
            response_cls=ACLInfoResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from typing import Dict, List
from tcsoa.gen.Internal.IcsAw._2018_05.Classification import PropertyInfo, ClassDefinition, ClassParents, KeyLOVDef, ClassChildren
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class BlockData(TcBaseObj):
    """
    Structure representing the list of blocks&rsquo; information of a Classification Object.
    
    :var blocks: A list of block information.
    """
    blocks: List[BlockInfo] = ()


@dataclass
class ClassificationObjectInfo2(TcBaseObj):
    """
    Structure representing Classification Object details.
    
    :var clsObject: The found Classification object.
    :var workspaceObject: The WorkspaceObject that is associated by this Classification object. If this is NULLTAG,
    then the found "clsObject" is a standalone classification object.
    :var properties: List of properties containing, attributes ids and their values.
    :var blockDataMap: A map (string, blockData) of block IRDI to the list of block data. IRDI comprises of
    Registration Authority Identifier, Data Identifier, separator and version identifier. Example of an block IRDI is
    "SPLM-0#01-000806#001".
    """
    clsObject: BusinessObject = None
    workspaceObject: WorkspaceObject = None
    properties: List[PropertyInfo] = ()
    blockDataMap: BlockDataMap = None


@dataclass
class ClassificationObjects2(TcBaseObj):
    """
    Structure representing the Classification Objects of a WorkspaceObject.
    
    :var clsObjects: A list of Classification Objects' information.
    """
    clsObjects: List[ClassificationObjectInfo2] = ()


@dataclass
class BlockInfo(TcBaseObj):
    """
    Structure containing classification block details. Classification block is a class attribute (Cst0ClassAttrute)
    pointing to property definition (Cst0PropertyDefinition) instance which references another classification class
    (Cst0ClassDefinition).
    
    :var classId: The classification class IRDI to which current block ID references. This is used in polymorphic
    blocks.
    :var owningBlocks: List of classification block IRDI to which given block belongs.
    :var cardinalityIndex: The index of a block in a given cardinal structure. Classification blocks can be configured
    as cardinal block in a class; this index is used to identify the location of the block in a given cardinal
    structure.
    :var properties: List of properties containing, attributes ids and their values.
    :var blockDataMap: A map (string, blockData) of block IRDI to the list of block data.
    """
    classId: str = ''
    owningBlocks: List[str] = ()
    cardinalityIndex: int = 0
    properties: List[PropertyInfo] = ()
    blockDataMap: BlockDataMap = None


@dataclass
class FindClassificationInfo2Response(TcBaseObj):
    """
    Structure representing the classification information returned by FindClassificationInfo2 operation.
    
    :var clsObjectDefs: A map (WorkspaceObject, ClassificationObjects2) of workspace objects to a list of
    Classification objects.
    :var keyLOVDescriptors: A map (string, KeyLOVDef) of Classification KeyLOV IDs/IRDIs (International Registration
    Data Identifier) and its definitions pairs.
    :var clsClassDescriptors: A map (string, ClassDefinition) of Classification classes or view IDs/IRDIs and its
    corresponding definition pairs. This map also contains class attribute information.
    :var classParents: A map (string, ClassParents) of Classification classes or view IDs/IRDIs and its parents'
    definition pairs.
    :var classChildren: A map (string, ClassChildren) of Classification classes or view IDs/IRDIs and its children's
    definition pairs.
    :var clsBlockDescriptors: A map (string, ClassDefinition) of Classification classes IRDIs and its corresponding
    definition pairs. This map also contains class attribute information. This map is only used for Classification
    Standard Taxonomy (CST) property block information. This map differs from clsClassDescriptors as it contains the
    information for property blocks that belong to the returned classes.
    :var serviceData: Any failures will be returned in the service data list of partial errors.
    """
    clsObjectDefs: ClassificationObjectsDefMap2 = None
    keyLOVDescriptors: KeyLOVDefMap2 = None
    clsClassDescriptors: ClassDefinitionMap2 = None
    classParents: ClassParentsMap2 = None
    classChildren: ClassChildrenMap2 = None
    clsBlockDescriptors: BlockDefinitionMap2 = None
    serviceData: ServiceData = None


@dataclass
class SaveClassificationObjects2Response(TcBaseObj):
    """
    Holds the classification objects returned by the saveClassificationObjects2 operation.
    
    :var classificationObjects: A list of created or updated Classification objects.
    :var serviceData: Any failures will be returned in the service data list of partial errors.
    """
    classificationObjects: List[ClassificationObjectInfo2] = ()
    serviceData: ServiceData = None


"""
A map (string, blockData) of block IRDI to the list of block data.
"""
BlockDataMap = Dict[str, BlockData]


"""
The list of Classification Class or View ID / IRDIs and corresponding definition.
"""
ClassDefinitionMap2 = Dict[str, ClassDefinition]


"""
A map of classification class Id / Irdi to a list of its parents.
"""
ClassParentsMap2 = Dict[str, ClassParents]


"""
Map of workspace object to a list of its classification objects or standalone classification objects with a matching ID to that of the WorkspaceObject. If an object is classified then map value will have all its classification's information. If object is not classified, but system found standalone classification objects, then the map values will have standalone classification object's information. In the case of a standalone classification, the standalone classification&rsquo;s WorkspaceObject will be a NULLTAG.
"""
ClassificationObjectsDefMap2 = Dict[WorkspaceObject, ClassificationObjects2]


"""
The list of KeyLOV ids and corresponding KeyLOV definition. KeyLOV id and definition will be stxt business object if the class is a from traditional classification, or from Cst0KeyLOVDefinition business object if it is from Classification Standard Taxonomy (CST).
"""
KeyLOVDefMap2 = Dict[str, KeyLOVDef]


"""
A map of classification class Id / IRDI to a list of block property class definitions. Used by Classification Standard Taxonomy (CST) when a class has block properties.
"""
BlockDefinitionMap2 = Dict[str, ClassDefinition]


"""
A map of classification class Id / Irdi to a list of its children.
"""
ClassChildrenMap2 = Dict[str, ClassChildren]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Dataset
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FileTicket(TcBaseObj):
    """
    The FileTicket struct.
    
    :var fileName: The file name.
    :var ticket: The FMS download ticket.
    """
    fileName: str = ''
    ticket: str = ''


@dataclass
class GetTemplateInput(TcBaseObj):
    """
    This structure defines the list of search criteria used to find the appropriate DMTemplates for the calling
    application.
    
    :var application: The name of the calling application.
    :var version: The version of the application. Optional.
    :var templateType: An application defined type value for different templates.
    :var templateUnits: The measurement units that the application uses.
    :var createItemType: The item Type to be created using the template files.
    :var nameReferenceName: The named reference name for the thumbnail file.
    :var relation: The relation to the thumbnail dataset.
    """
    application: str = ''
    version: str = ''
    templateType: str = ''
    templateUnits: str = ''
    createItemType: str = ''
    nameReferenceName: str = ''
    relation: str = ''


@dataclass
class GetTemplateOutput(TcBaseObj):
    """
    The GetTemplateOutput struct.
    
    :var templateRev: The document template revision.
    :var templateName: The document template name.
    :var dataset: The thumbnail dataset associated with the template.
    :var fileTicket: The file ticket for the FMS download of the thumbnail file.
    """
    templateRev: BusinessObject = None
    templateName: str = ''
    dataset: Dataset = None
    fileTicket: FileTicket = None


@dataclass
class GetTemplatesResponse(TcBaseObj):
    """
    The GetTemplateOutput struct.
    
    :var output: The list of references to struct GetTemplateOutput.
    :var serviceData: The Service Data. Partial errors and failures are updated and returned through this object.
    """
    output: List[GetTemplateOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AlignmentMismatches(TcBaseObj):
    """
    This structure provides information about a pair of mismatched lines.
    
    :var mismatchedAlignment: the pair of mismatched BOM lines
    :var mismatchedproperties: the indices of mismatched properties
    """
    mismatchedAlignment: BOMLinePair = None
    mismatchedproperties: List[int] = ()


@dataclass
class BOMLinePair(TcBaseObj):
    """
    This structure provides a pair of BOM lines.
    
    :var source: the source BOM line
    :var target: the target BOM line
    """
    source: BOMLine = None
    target: BOMLine = None


@dataclass
class AlignmentCheckResponse(TcBaseObj):
    """
    This structure provides response of the call.
    
    :var propertyList: the property list used in this alignment check
    :var results: the mismatched BOM lines
    :var serviceData: Partial errors
    """
    propertyList: List[str] = ()
    results: List[AlignmentCheckResult] = ()
    serviceData: ServiceData = None


@dataclass
class AlignmentCheckResult(TcBaseObj):
    """
    This structure provides information about the alignment check in a given scope.
    
    :var scope: the pair of scope BOM lines
    :var mismatches: the mismatched BOM lines
    """
    scope: BOMLinePair = None
    mismatches: List[AlignmentMismatches] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Item, ItemRevision
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from typing import Dict


@dataclass
class IdInfo2(TcBaseObj):
    """
    A Teamcenter object structure that contains the related information that need to be visualized in Teamcenter
    Visualization.
    
    :var object: The business object to be laucnhed. Launched object could be Item, ItemRevision, Dataset,
    BOMView_Revision, BOMView. The business object will be resolved by the server in some cases (e.g. Item or
    ItemRevision launch) to a directly launchable visualization object (such as a DirectModel Dataset or
    BOMViewRevision).
    :var item: The parent or containing Item of the launched object (object).  If this is not known, the server will
    attempt to identify the parent if it can.  If the parent information cannot be identified, this will not result in
    an error condition, it simply will not pass the information to the visualization client which for some cases may
    affect what features are available in the client.
    :var itemRev: The parent or containg ItemRevision of the launched object (id). If this is not known, the server
    will attempt to identify the parent if it can.
    :var operation: The type of launch action the viewer is instructed to perform (e.g. open, insert, merge, etc).  If
    left empty, it will default to open. This controls the action the viewer will perform when it opens the object. The
    following actions are supported: 
    1. open - Open the object in a new window. 
    2. Insert - Insert the object into the current window that has focus.
    3. merge - Attempt to merge a pruned product structure with one that is already open (and has focus) if it can.
    4. Interop -Present a dialog that lets the user select the launch action.
    :var idAdditionalInfo: This is a generic mechanism for putting additional key value pairs into the VVI file that
    control viewer launch behavior.
    :var structureMode: This specifies the structure launch mode to open the structure object in Teamcenter
    Visualization. The different structure launch modes that are supported are:
    1. Dynamic - When this mode is used, it launches the dynamic structure in Teamcenter Visualization.
    2. Static - When this mode is used, it launches the static structure in Teamcenter Visualization.
    3. Preference - This specifies that Teamcenter Visualization preference is used to launch the structure.
    4. Ask - This mode of launch asks user to specify the launch mode.
    :var clientId: This is the UID of the launched object against which the VVI string buffer will be stored.
    """
    object: BusinessObject = None
    item: Item = None
    itemRev: ItemRevision = None
    operation: str = ''
    idAdditionalInfo: KeyValueMap2 = None
    structureMode: str = ''
    clientId: str = ''


@dataclass
class LaunchInfoResponse(TcBaseObj):
    """
    A response structure for the createLaunchInfo () operation.
    
    :var ticket: The VVI file ticket to the transient volume.
    This member is valid only if  hasTransientVolume flag is set to true in SessionInfo2.
    
    :var vviStrBuffersOutputMap: The map of clientId to the VVI string buffer(stirng/string). This member is valid only
    if hasTransientVolume flag is set to false in SessionInfo2.
    :var serviceData: The object containing error information. In cases where objects can't be launched, error message,
    codes will be mapped to respective object id in the list of partial errors.
    """
    ticket: str = ''
    vviStrBuffersOutputMap: VVIStringBufferOutputMap = None
    serviceData: ServiceData = None


@dataclass
class SessionInfo2(TcBaseObj):
    """
    Object containing session information for the viewer to connect to the session.  Includes the session discriminator
    and any other additional session relevant key value pair.
    
    :var sessionDescriminator: Client-Server session discriminator to connect to an existing tcserver session.  This
    allows the viewer to share an existing tcserver.exe session with the launching client. This should be the same as
    the SessionDescriminator argument of the SessionService.login operaiton.  If this empty string, the viewer will
    present a login prompt if it does not already have a session established.
    :var sessionAdditionalInfo: The additional information related to session to put in the VVI file/buffer in the form
    of key value pair (key (string) = value(/stirng)).(Refer to Teamcenter Visualization VVI documentation on
    acceptable additional information).
    :var hasTransientVolume: If true the VVI information is returned as a file ticket to the transient volume,
    otherwise it is returned directly as a string.
    """
    sessionDescriminator: str = ''
    sessionAdditionalInfo: KeyValueMap2 = None
    hasTransientVolume: bool = False


"""
Map of key-value name pair.
"""
KeyValueMap2 = Dict[str, str]


"""
A map containing client ids or additional Teamcenter object UID  and vviStringBuffer as key/value pairs.
"""
VVIStringBufferOutputMap = Dict[str, str]



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class NameAuthorizationList(TcBaseObj):
    """
    This structure holds the accessibility information for the given user and group combinations on the given
    applications or utilities.
    
    :var output: A list of NameAuthorizationResponse objects one for each of the given application or utility.
    :var partialErrors: Object that holds the partial errors that occurred while getting the accessibility on any of
    the given application or utility.
    """
    output: List[NameAuthorizationResponse] = ()
    partialErrors: ServiceData = None


@dataclass
class NameAuthorizationResponse(TcBaseObj):
    """
    This structure contains the name of the given application or utility, rule domain name and the derived verdict that
    indicates whether the user and group combination have accessibility on the application or utility or not.  True
    value for the verdict indicates the user have authorization access on the application or the utility. False value
    for the verdict indicates the user doesn't have authorization access on the application or the utility.
    
    :var name: Name of the application or the utility.
    :var ruleDomain: Rule domain name.
    :var verdict: Derived verdict for the given user and group combination on the application or utility.
    """
    name: str = ''
    ruleDomain: str = ''
    verdict: bool = False


@dataclass
class NameInfo(TcBaseObj):
    """
    This structure contains the name of the application or utility and the rule domain that indicates whether the name
    is that of an application or that of a utility.
    
    :var name: Name of an application or a utility whose accessibility for the given user need to be determined.
    :var ruleDomain: The name of the rule domain. Valid rule domain  name is either Utility or Application.
    """
    name: str = ''
    ruleDomain: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ValidationCheckCallbackParam(TcBaseObj):
    """
    Validation Check Callback Param
    
    :var type: The type of the validation always MFG_ValidationChecksCallback
    :var library: Customized DLL Name
    :var name: Customized Name - The name of the validation test
    :var func: Customized Function name
    :var failAllOnError: Continue to the next test if the previous test failed.
    """
    type: str = ''
    library: str = ''
    name: str = ''
    func: str = ''
    failAllOnError: bool = False


@dataclass
class ValidationCheckExecutionErrors(TcBaseObj):
    """
    Validation Check Execution Errors
    
    :var object: the non valid object line
    :var validationErrors: validationErrors
    :var validationTest: The validation check that failed.
    """
    object: BusinessObject = None
    validationErrors: List[ValidationCheckExecutionErrorsDetails] = ()
    validationTest: ValidationCheckCallbackParam = None


@dataclass
class ValidationCheckExecutionErrorsDetails(TcBaseObj):
    """
    Validation Check Execution Errors Details
    
    :var msgId: could be ifail or lov (msgId from ValidationNotice struct)
    :var msg: description of the localized error
    :var object: The object on which the validation check failed
    :var validationNoticeType: The type of the returned notice from the callback functions
    """
    msgId: int = 0
    msg: str = ''
    object: BusinessObject = None
    validationNoticeType: str = ''


@dataclass
class ValidationCheckExecutionParam(TcBaseObj):
    """
    Validation Check Execution Param
    
    :var root: The root object to be validated, could be any line type of Mfg0BvrProcess or Mfg0BvrOperation or its
    derived.
    :var validationChecks: all the validation checks to perform on the object
    """
    root: BusinessObject = None
    validationChecks: List[ValidationCheckCallbackParam] = ()


@dataclass
class ValidationsChecksExecutionResponse(TcBaseObj):
    """
    Validations Checks Execution Response
    
    :var errors: errors
    :var serviceData: Standard ServiceData member
    """
    errors: List[ValidationCheckExecutionErrors] = ()
    serviceData: ServiceData = None


@dataclass
class ValidationsChecksObjectsResponse(TcBaseObj):
    """
    Validations Checks Objects Response
    
    :var params: params
    :var serviceData: serviceData
    """
    params: List[ValidationCheckCallbackParam] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from typing import List
from tcsoa.gen.BusinessObjects import SchTaskDeliverable, ScheduleTask, SchDeliverable, WorkspaceObject
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class TaskDeliverableContainer(TcBaseObj):
    """
    The input information for a single task deliverable.
    
    :var scheduleTask: The task which to which the new delievable will be associated to.
    :var submitType: Specifies the type of the workflow submittal to which the deliverable is routed. Valid values are
    3=Don't submit, 0=submit as target, 1=submit as reference.
    :var deliverableReference: The instance of the Item, Form, Folder Dataset and Document using which the
    SchDeliverable and SchTaskDeliverable are createad.
    :var deliverableName: The name of the deliverable.
    :var deliverableType: The type of the instance associated with the deliverable (Item, Form, Folder Dataset and
    Document).
    """
    scheduleTask: ScheduleTask = None
    submitType: int = 0
    deliverableReference: WorkspaceObject = None
    deliverableName: str = ''
    deliverableType: str = ''


@dataclass
class TaskDeliverablesResponse(TcBaseObj):
    """
    The response container for CreatedTaskDeliverable. CreatedScheduleDeliverable, ScheduleTask that contains
    Delieverable.
    
    :var scheduleTask: The ScheduleTask that is associated with the deliverable.
    :var scheduleDeliverable: The created ScheduleDeliverable to reference. Tag of ScheduleDeliverable.
    :var scheduleTaskDeliverable: The ScheduleTaskDeliverable to reference. Tag of scheduleTask deliverable.
    """
    scheduleTask: ScheduleTask = None
    scheduleDeliverable: SchDeliverable = None
    scheduleTaskDeliverable: SchTaskDeliverable = None


@dataclass
class CreateTaskDeliverablesResponse(TcBaseObj):
    """
    The collection of individual TaskDeilverableResponse.
    
    :var responses: The list of individual Created ScheduleTaskDeliverable,Created Scheduledeliverable and Schedule
    task to which the ScheduleDeliverable and ScheduleTaskDeliverable are associated.
    :var serviceData: The service data.
    """
    responses: List[TaskDeliverablesResponse] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.base import TcService
from tcsoa.gen.Internal.Core._2007_09.DataManagement import WhereUsedResponseOccGroup


class DataManagementService(TcService):

    @classmethod
    def whereUsedOccGroup(cls, objects: List[BusinessObject], numLevels: int, whereUsedPrecise: bool, rule: BusinessObject) -> WhereUsedResponseOccGroup:
        """
        Finds the assemblies that a given Occurrence Group is used (unpublished for the case of Occurrence Group).
        """
        return cls.execute_soa_method(
            method_name='whereUsedOccGroup',
            library='Internal-Core',
            service_date='2007_09',
            service_name='DataManagement',
            params={'objects': objects, 'numLevels': numLevels, 'whereUsedPrecise': whereUsedPrecise, 'rule': rule},
            response_cls=WhereUsedResponseOccGroup,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class MoveAndResequenceParameter(TcBaseObj):
    """
    The structure specifies the nodes to be moved, their target, whether dropped on the target, context based on which
    find number to be decided and whether to clone or move the nodes to the new target.
    
    :var sourceNodes: A list of BOMLine objects to be re-sequenced within parent or moved to another target BOMLine.
    :var targetNode: The BOMLine to which the source BOMLine objects are to be moved.
    :var predecessor: The BOMLine below which source BOMLine objects are dropped. The find number of dropped lines is
    altered to position them after the predecessor. The find number of siblings following the pasted BOMLine objects
    may also change. Ignored if the parameter isDroppedOnTarget is true which signifies that source BOMLine objects are
    directly dropped on the target BOMLine, as a result find numbers are generated for dropped BOMLine objects and the
    BOMLine objects are position as the last children in the BOM structure.
    :var isDroppedOnTarget: Specifies whether source BOMLine objects are directly dropped on the target node. If true,
    the find numbers are generated for dropped BOMLine objects and the BOMLine objects are position as the last
    children in the BOM structure. If false, find numbers are generated so as to position them after the predecessor.
    However, in case predecessor is NULL then the find numbers are generated for dropped BOMLine objects and BOMLine
    objects are position as the first children in the BOM structure.
    :var findNumberContext: Context based on which find numbers of dropped BOMLine objects are generated. Relevant only
    if the target BOMLine is process resource of type Mfg0BvrProcessResource.
    :var isPasteDuplicate: Specifies whether the source BOMLine objects are to be cloned to the new target BOMLine
    instead of move. If true, source BOMLine objects are cloned to the target BOMLine objects. If false, the source
    BOMLine objects are moved to the target BOMLine.
    """
    sourceNodes: List[BusinessObject] = ()
    targetNode: BusinessObject = None
    predecessor: BusinessObject = None
    isDroppedOnTarget: bool = False
    findNumberContext: BusinessObject = None
    isPasteDuplicate: bool = False


@dataclass
class MoveAndResequenceResponse(TcBaseObj):
    """
    response structure for the service method moveAndResequenceNodes.
    
    :var createdICRevs: List of newly created Incremental Change (IC) revisions.
    :var createdFutureICRevs: List of newly created future IC revisions.
    :var newChildLines: The map (BOMLine, List of BOMLine) of target BOMline and the newly created or moved BOMLine
    objects under it.
    :var serviceData: The service data containing partial errors.
    """
    createdICRevs: List[WorkspaceObject] = ()
    createdFutureICRevs: List[WorkspaceObject] = ()
    newChildLines: TargetToNewChildLinesMap = None
    serviceData: ServiceData = None


"""
The map of target BOMline and the newly created or moved BOMLine objects under it.
"""
TargetToNewChildLinesMap = Dict[BusinessObject, List[BusinessObject]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMWindow, ItemRevision
from tcsoa.gen.Internal.StructureManagement._2009_10.EffectivitiesManagement import EffectivityGroupInputInfo, EffectivitiesInputInfo, GetEffectivityGrpListResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class EffectivitiesManagementService(TcService):

    @classmethod
    def setEndItemEffectivityGroups(cls, effectivityGrpInput: List[EffectivityGroupInputInfo]) -> ServiceData:
        """
        Apply a list of Fnd0EffectvtyGrpRevision objects to a BOMWindow
        
        Use cases:
        User selects a structure with occurrence Effectivity defined and opens it in a BOMWindow .  This structure can
        then be configured by applying a list of Fnd0EffectvtyGrpRevision objects to the BOMWindow. This use case
        requires Multi Unit Configuration to be enabled at the site.
        """
        return cls.execute_soa_method(
            method_name='setEndItemEffectivityGroups',
            library='Internal-StructureManagement',
            service_date='2009_10',
            service_name='EffectivitiesManagement',
            params={'effectivityGrpInput': effectivityGrpInput},
            response_cls=ServiceData,
        )

    @classmethod
    def createOrUpdateEffectivites(cls, effectivitiesInfo: List[EffectivitiesInputInfo], effectivityGroupRevision: ItemRevision) -> ServiceData:
        """
        Creates or updates effectivity information for the given Fnd0EffectvtyGrpRevision represented by the input
        parameter 'effectivityGroupRevision'.
        
        Use cases:
        - Create Effectivity objects for a given Fnd0EffectvtyGrpRevision  object.
        - Update Effectivity information like unit range and end item for a list of Effectivity objects belonging to an
        Fnd0EffectvtyGrpRevision
        - Remove an Effectivity object from an Fnd0EffectvtyGrpRevision object.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateEffectivites',
            library='Internal-StructureManagement',
            service_date='2009_10',
            service_name='EffectivitiesManagement',
            params={'effectivitiesInfo': effectivitiesInfo, 'effectivityGroupRevision': effectivityGroupRevision},
            response_cls=ServiceData,
        )

    @classmethod
    def getEffectivityGrpRevList(cls, bomWindow: BOMWindow) -> GetEffectivityGrpListResponse:
        """
        Gets the list of Fnd0EffectvtyGrpRevision objects applied on the given BOMWindow
        
        Use cases:
        User would like to retrieve the list of Fnd0EffectvtyGrpRevision objects applied to a BOMWindow
        """
        return cls.execute_soa_method(
            method_name='getEffectivityGrpRevList',
            library='Internal-StructureManagement',
            service_date='2009_10',
            service_name='EffectivitiesManagement',
            params={'bomWindow': bomWindow},
            response_cls=GetEffectivityGrpListResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, NamedVariantExpression, ItemRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetMultipleNVEResponse(TcBaseObj):
    """
    This structure contains the list of created NVEs and 'ServiceData' object.
    
    :var aNves: List of newly created NamedVariantExpressions. One for each successfully created NVEs.
    :var serviceData: The 'ServiceData' object containing full/partial errors.
    """
    aNves: List[NamedVariantExpression] = ()
    serviceData: ServiceData = None


@dataclass
class GetSVRInfo(TcBaseObj):
    """
    The 'GetSVRInfo' structure refers to the object containing information about Saved Variant Rule.
    
    :var itemRev: Reference to the ItemRevision object of the product context on which SVR is to be created
    :var svrData: List of 'SVRData' objects which contains the information like SVR name, description, relation, option
    value pair.
    """
    itemRev: ItemRevision = None
    svrData: List[SVRData] = ()


@dataclass
class GetSVRResponse(TcBaseObj):
    """
    This is a structure that has references of all the created/modified SVRs in the operation along with the
    'ServiceData'.
    
    :var aSVRs: A collection of all the created/modified SVR(s).
    :var serviceData: An object of 'ServiceData' which contains updated data model objects and/or error objects with
    error messages.
    """
    aSVRs: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class MultipleNamedVariantExpressions(TcBaseObj):
    """
    This structure contains the information required to create multiple NVEs. It contains the Architecture Breakdown
    revision object, a boolean flag and list of 'NamedVariantExpressionsInfo' structure.
    
    :var archRev: Reference to the Architecture Breakdown revision object. If this reference is not supplied then NVE
    will not be created.
    
    :var ignoreDuplicates: Boolean value to indicate whether duplicate NVE should be checked or not. 
    If it is false then uniqueness of NVE will be checked.
    If it is true then uniqueness of NVE will not be checked then multiple NVEs can be created with the same variant
    condition.
    
    
    :var nveInfo: List of 'NamedVariantExpressionsInfo' objects. Each  object contains the information required to
    create a Named Variant Expression.
    """
    archRev: ItemRevision = None
    ignoreDuplicates: bool = False
    nveInfo: List[NamedVariantExpressionsInfo] = ()


@dataclass
class NamedVariantExpressionsInfo(TcBaseObj):
    """
    This structure contains the information required to create a NVE. It contains name, description and
    VariantCondition.
    
    :var code: Name of the NamedVariantExpression. When shared NVE is used it is mandatory to supply the name. If code
    is not supplied then NVE will be created without any name. These kinds of NVE cannot be shared.
    :var desc: Description of the NamedVariantExpression. It is not mandatory. If description is not supplied NVE will
    be created without description.
    :var varCondition: Variant Conditions for the NamedVariantExpression. It is required that a valid variant condition
    is supplied for this operation to succeed.
    """
    code: str = ''
    desc: str = ''
    varCondition: BusinessObject = None


@dataclass
class SVRData(TcBaseObj):
    """
    The 'SVRData' structure contains the information about Saved Variant Rule(s) which are to be created.
    
    :var varRule: Specifies the default BOMVariantRule. If the SVR being created already exists then this default
    variant rule is applied on existing rule.
    :var name: Specifies the name of SVR
    :var desc: Specifies the description of SVR
    :var relation: Specifies the relation with which SVR will be attached to the provided ItemRevision object
    :var option: List of options to be set in the variant rule
    :var value: List of values corresponding to the supplied options. This attribute is used in conjunction with the
    option attribute, i.e. option value pair
    """
    varRule: BusinessObject = None
    name: str = ''
    desc: str = ''
    relation: str = ''
    option: List[BusinessObject] = ()
    value: List[str] = ()


@dataclass
class DeleteNamedVariantExpressions(TcBaseObj):
    """
    This structure contains architecture breakdown and the list of NVEs to be deleted.
    
    :var archRev: Reference to the Architecture Breakdown revision object. If this reference is not supplied then this
    operation will fail.
    :var code: List of the Named Variant Expressions to be deleted.
    """
    archRev: ItemRevision = None
    code: List[NamedVariantExpression] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dataset
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExportObjectsToOfflinePackageResponse(TcBaseObj):
    """
    ExportObjectsToOfflinePackageResponse structure defines the response from ExportObjectsToOfflinePackage operation.
    It contains briefcase file FMS ticket, export log file FMS ticekts, briefcase dataset, and partial errors.
    
    :var briefcaseFileFMSTicket: FMS ticket of the briefcase file, which can be used to download the briefcase file
    from server to client.
    :var exportLogFileFMSTickets: FMS ticket of the briefcase file, which can be used to download the briefcase file
    from server to client.
    :var briefcaseDataSet: A Dataset which includes the out briefcase file in its namedReference.
    After export, a new Dataset will be created. The exported briefcase file will be added to the new Dataset. And a
    new mail which contains the Dataset will be added to mailbox folder of the user.  The mail will also be send to
    mail box of the user if the user has set the Email Address.
    
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors listed
    above in case of failure conditions.
    """
    briefcaseFileFMSTicket: str = ''
    exportLogFileFMSTickets: List[str] = ()
    briefcaseDataSet: Dataset = None
    serviceData: ServiceData = None


@dataclass
class ImportObjectsFromOfflinePackageResponse(TcBaseObj):
    """
    ImportObjectsFromOfflinePackageResponse structure defines the response from ImportObjectsFromOfflinePackageResponse
    operation. It contains FMS ticket of the log file, error file, and partial errors and objects that are imported.
    
    :var logFileFMSTicket: FMS ticket of the import log file, which can be used to download the import log file from
    server to client.
    :var errorFileFMSTicket: FMS ticket of the import error file, which can be used to download the import error file
    from server to client.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors listed
    above in case of failure conditions.
    """
    logFileFMSTicket: str = ''
    errorFileFMSTicket: str = ''
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.Core._2011_06.ICT import InvokeICTMethodResponse, Arg
from typing import List
from tcsoa.base import TcService


class ICTService(TcService):

    @classmethod
    def invokeICTMethod(cls, className: str, methodName: str, args: List[Arg]) -> InvokeICTMethodResponse:
        """
        Invoke ICT method.
        """
        return cls.execute_soa_method(
            method_name='invokeICTMethod',
            library='Internal-Core',
            service_date='2011_06',
            service_name='ICT',
            params={'className': className, 'methodName': methodName, 'args': args},
            response_cls=InvokeICTMethodResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class PrintSubmitRequestInfo(TcBaseObj):
    """
    The input values needed to submit the print request.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var printObjs: A list of objects that will get printed.
    :var collate: When two or more copies are printed, this specifies whether the printed pages are collated.
    :var printToScale: Specifies the scaling factor.
    :var orientation: Specifies the paper orientation of best fit, portrait or landscape.
    :var bannerPage: Specifies whether to print a page including the defined stamps and listing additional data as
    specified by the VVCP setup.
    :var extraInfo: The placeholder for extra name value pair information.
    :var printerConfigurationName: The PrintConfiguration object name.
    :var printerName: The printer name.
    :var colorMode: The print color mode.
    :var userStamp: Specifies text for a user stamp to be applied in addition to any existing system stamp
    configuration.
    :var paperSize: The print paper size.
    :var printStamp: Specify whether the print stamp applies to the first page, the banner page, or all pages.
    :var pageRange: Specifies a range of pages to print.
    :var numberCopies: The number of copies.
    """
    clientId: str = ''
    printObjs: List[BusinessObject] = ()
    collate: bool = False
    printToScale: str = ''
    orientation: str = ''
    bannerPage: str = ''
    extraInfo: AttributeMap = None
    printerConfigurationName: str = ''
    printerName: str = ''
    colorMode: str = ''
    userStamp: str = ''
    paperSize: str = ''
    printStamp: str = ''
    pageRange: str = ''
    numberCopies: str = ''


@dataclass
class PrinterDefinitionOutput(TcBaseObj):
    """
    Printer definition output, this information will be used for the printSubmitRequest operation.
    
    :var printerConfigurationName: The PrintConfiguration object name.
    :var printerName: The printer name.
    :var paperSizes: List of paper sizes.
    :var supportStamp: Support stamp or not.
    :var printableDatasetTypeNames: The printable dataset type names.
    """
    printerConfigurationName: str = ''
    printerName: str = ''
    paperSizes: List[str] = ()
    supportStamp: bool = False
    printableDatasetTypeNames: List[str] = ()


@dataclass
class PrinterDefinitionResponse(TcBaseObj):
    """
    The return list of PrinterDefinitionOutput structures.
    
    :var outputs: List of PrinterDefinitionOutput.
    :var serviceData: Standard return; includes any error information.
    """
    outputs: List[PrinterDefinitionOutput] = ()
    serviceData: ServiceData = None


@dataclass
class RenderSubmitRequestInfo(TcBaseObj):
    """
    The input values needed to submit the render request.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var renderObjs: A list of objects that will get rendered.
    :var preserve: If true preserve the previous rendered dataset, otherwise replace the rendered dataset.
    :var extraInfo: The placeholder for extra name value pair information.
    """
    clientId: str = ''
    renderObjs: List[BusinessObject] = ()
    preserve: bool = False
    extraInfo: AttributeMap = None


@dataclass
class SubmitRequestOutput(TcBaseObj):
    """
    The structure contains the created request object.
    
    :var clientId: The unmodified value from the PrintSubmitRequestInfo.clientId. This can be used by the caller to
    identify this data structure with the source input data.
    :var requestObject: The request object created.
    """
    clientId: str = ''
    requestObject: BusinessObject = None


@dataclass
class SubmitRequestResponse(TcBaseObj):
    """
    The structure contains a list of SubmitRequestOutput.
    
    :var outputs: List of SubmitPrintRequestOutput.
    :var serviceData: Standard return; includes any error information.
    """
    outputs: List[SubmitRequestOutput] = ()
    serviceData: ServiceData = None


"""
String to string map, it is extra information for the request
"""
AttributeMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ImanItemBOPLine
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class SearchScopedStructureInputInfo(TcBaseObj):
    """
    A search input consists of list of process line object from BOP structure on which the search is executed, the
    object type to be searched for, the query type of the search, the closure rule to traverse the BOP structure , a
    flag to return the object type which are assigned to the input process line object and a map containing the
    additional information (if required).
    
    :var processScopeForSearch: The list of process line object from BOP structure, which has defined scope from Bill
    of Material (BOM).
    :var objectType: The object type to be searched for. Valid types are: Mfg0BvrWeldPoint, Mfg0BvrArcWeld, Mfg0BvrPLP
    and Mfg0BvrPart.
    :var queryType: The query to search on the input process scope. The valid values are: "Unassigned",
    "ConsumedConnected" and "ConsumedConnectedUnassigned". The "Unassigned" query searches for the unassigned input
    object type in the process scope specified as input. The "ConsumedConnected" query first searches for the assigned
    input object type in the input process scope and then for output objects, it finds out the connected parts. The
    "ConsumedConnectedUnassigned" query first searches for the assigned input object type in the input process scope
    and then for output objects, it finds out the connected parts and filter out unassigned parts.
    :var closureRuleName: The name of the closure rule to be used to traverse the BOP structure.
    :var returnAssignedObjects: If true, the response also returns input object type which are assigned to the input
    process scope.
    :var additionalInfo: A structure containing the maps to capture the additional information.
    1. User can provide the additional scope for the query which is not defined by "Set Scope" operation. The key is
    "AdditionalScope" and the value is list of additional scope from the BOM structure.
    2. User need to search for the objects of specific type which are not part of the BOM scope. The key is
    "OutOfScopeObjects" and the value is the "Object Type".
    """
    processScopeForSearch: List[ImanItemBOPLine] = ()
    objectType: BusinessObject = None
    queryType: str = ''
    closureRuleName: str = ''
    returnAssignedObjects: bool = False
    additionalInfo: AdditionalInfo2 = None


@dataclass
class SearchScopedStructureResponse(TcBaseObj):
    """
    Returns a list of object which is result of search query, a list of object which is part of input process scope but
    not part of BOM scope, a list of objects which is assigned to input process scope and partial errors in the service
    data.
    
    :var queryOutput: A list of object which is result of search query based on the input criteria.
    :var outOfScopeObjects: A list of object which is part of the input process scope from Bill Of Process (BOP)
    structure but not part of the product scope from Bill Of Material (BOM) structure.
    :var assignedObjects: A list of object which is assigned to the input process line object in Bill Of Process (BOP)
    structure. This list is populated only if the logical value of "returnAssignedObjects" in the input is "true".
    :var serviceData: A service data containing partial errors.
    """
    queryOutput: List[BusinessObject] = ()
    outOfScopeObjects: List[BusinessObject] = ()
    assignedObjects: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class AdditionalInfo2(TcBaseObj):
    """
    A structure containing the maps to capture the additional information.
    
    :var strToDateMap: A map(string, list of date) to capture additional information.
    :var strToDoubleMap: A map(string, list of double)  to capture additional information.
    :var strToStrMap: A map(string, list of string)  to capture additional information.
    :var strToIntMap: A map(string, list of integer)  to capture additional information.
    :var strToObjMap: A map(string, list of business object) to capture additional information.
    """
    strToDateMap: StringToDateMap = None
    strToDoubleMap: StringToDoubleMap = None
    strToStrMap: StringToStringMap = None
    strToIntMap: StringToIntegerMap = None
    strToObjMap: StringToObjectMap = None


"""
A map(string, list of date) to capture additional information.
"""
StringToDateMap = Dict[str, List[datetime]]


"""
A map(string, list of double)  to capture additional information.
"""
StringToDoubleMap = Dict[str, List[float]]


"""
A map(string, list of integer)  to capture additional information.
"""
StringToIntegerMap = Dict[str, List[int]]


"""
A map(string, list of business object) to capture additional information.
"""
StringToObjectMap = Dict[str, List[BusinessObject]]


"""
A map(string, list of string)  to capture additional information.
"""
StringToStringMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExportObjectsForTranslationResponse(TcBaseObj):
    """
    This structure holds the file tickets for the generated XML files as well as any partial errors.
    
    :var data: This holds the partial errors, for which XML data could not be generated. Each partial error has the
    offending business object.
    :var tickets: A list of FMS transient file tickets.
    """
    data: ServiceData = None
    tickets: List[LocaleTicketInfo] = ()


@dataclass
class LocaleTicketInfo(TcBaseObj):
    """
    Holds the locale and the corresponding file ticket for the generated XML file.
    
    :var locale: Locale-  The valid locale name should be in the format as outlined in the Java Standard Language - for
    instance "fr_FR" for French or "en_US" for English.
    :var ticket: The transient file ticket to be used for downloading the generated xml file.
    """
    locale: str = ''
    ticket: str = ''


@dataclass
class ObjectsInfo(TcBaseObj):
    """
    'ObjectsInfo' structure contains parameter required to filter Teamcenter business objects based on selected
    criteria that can be used to export localized business objects for review or translation purpose.
    
    :var translationsExist:  "true" value indicates review mode and "false" indicates translate mode.
     
    Review allows user to export all localized business objects into XML file based on status and locale selected. 
    
    Translate option exports business objects into XML file for selected target locale.
    
    :var locale: The name of the desired locale. The valid locale name should be in the format as outlined in the Java
    Standard Language (i.e. en_US for English, United States). A full list of locales supported by the Teamcenter
    server can be obtained from the service operation 'LanguageInformation.getLanguageList'.
    This can also be set to English word Master for exporting all business objects into XML file that needs to be
    translated to different locale. This parameter works in conjunction with "'translationsExist'" and "'statuses'". 
    
     If "'locale'" is set to Master, it indicates translate mode and valid values for other parameters:
           "'translationsExist'" must be "false", In case of value is set to "true", the 'ServiceData' contains error
    269002. "'statuses'" should be empty; this value is ignored for Master 'locale'.
    
    If "locale" is set to any other valid locale value then possible values for other parameters and behavior is:
          "'translationsExist'" is set to "true" indicate review mode. This will export all the business objects for
    selected "'statuses'". 
    
    :var statuses: A list of status to filter the localized values on. The status may be one or more of the following:
    Approved, In-Review, Pending or Invalid . This is ignored if mode is "translate" ('translationsExist' is false) and
    selected locale is Master.  In "review" mode nothing is exported in case of invalid "'statuses'" is passed and
    operation returns error 269002 in 'ServiceData'.
    :var transferMode: The name  of the Transfer Mode that is used while exporting objects. This is retrieved using
    'ImportExportImpl::GetTransferModesResponse ImportExportImpl::getTransferModes' operation.
    :var objects: A list of Teamcenter business objects which need to be filtered or exported based on the passed in
    parameters. Valid Teamcenter business objects are: Item, ItemRevision etc.
    :var typesPropertiesInfo: A list of type names and the corresponding properties.
    """
    translationsExist: bool = False
    locale: str = ''
    statuses: List[str] = ()
    transferMode: str = ''
    objects: List[BusinessObject] = ()
    typesPropertiesInfo: List[TypePropertiesInfo] = ()


@dataclass
class TransientFileTicketStatusInfo(TcBaseObj):
    """
    It holds the transient file ticket and the status for the translations in the file to be set in the DB.
    
    :var transientFileTicket: A valid FMS ticket to an XML file. This XML file is generated by language expert for
    business objects that need to be imported into system with status  as specified by "'localizationStatus'".
    :var localizationStatus: Localization status for the translations in the input XML file. The localization status
    can be one of the following: Approved, In-Review, Pending or Invalid. The default localization status is Approved.
    In case of empty or wrong status, the operation uses default status.
    """
    transientFileTicket: str = ''
    localizationStatus: str = ''


@dataclass
class TypePropertiesInfo(TcBaseObj):
    """
    TypePropertiesInfo structure holds the type name and the corresponding list of properties.
    
    :var typeName: Type name is a Teamcenter business object  type name for instance Item, Form etc.
    :var propertyNames: List of properties for the type.
    """
    typeName: str = ''
    propertyNames: List[str] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ReportDefinition
from tcsoa.gen.Reports._2015_03.CrfReports import GenerateReportsResponse2, PrintReportsCriteriaAsync, GeneratePrintReportsResponse, PrintReportsCriteria, GenerateReportsCriteria2Async
from tcsoa.gen.Reports._2010_04.CrfReports import GenerateReportsCriteria2
from tcsoa.gen.Reports._2007_01.CrfReports import GetReportDefinitionsResponse
from typing import List
from tcsoa.base import TcService


class CrfReportsService(TcService):

    @classmethod
    def generatePrintReports(cls, reportDefObj: ReportDefinition, inputs: List[PrintReportsCriteria]) -> GeneratePrintReportsResponse:
        """
        This operation generates the report based on the selected object and the report definition template.
        Optionally, it also sends the generated report to the physical printer for printing.
        """
        return cls.execute_soa_method(
            method_name='generatePrintReports',
            library='Reports',
            service_date='2015_03',
            service_name='CrfReports',
            params={'reportDefObj': reportDefObj, 'inputs': inputs},
            response_cls=GeneratePrintReportsResponse,
        )

    @classmethod
    def generatePrintReportsAsync(cls, reportDef: ReportDefinition, inputs: List[PrintReportsCriteriaAsync]) -> None:
        """
        This operation generates the print report asynchronously, based on the selected object and the report
        definition template. 
        Optionally, it also sends the generated report to the physical printer for printing.
        """
        return cls.execute_soa_method(
            method_name='generatePrintReportsAsync',
            library='Reports',
            service_date='2015_03',
            service_name='CrfReports',
            params={'reportDef': reportDef, 'inputs': inputs},
            response_cls=None,
        )

    @classmethod
    def generateReport(cls, inputs: List[GenerateReportsCriteria2]) -> GenerateReportsResponse2:
        """
        Generates reports (Summary Report/Custom Report/Item Report) using the specified criteria and the selected
        report style. The report will be displayed by the selected report style at the end. If no report style is
        selected, then the report will be displayed in xml file to the end user. If user would like to save the report
        as a Dataset, it will use the provided Dataset name to save the report to Teamcenter. After the report is
        generated, the report file will be uploaded to the transient volumes, user can get it from the transient
        volumes. Multiple reports generation is not supported currently. 
        
        Use cases:
        User can generate one report (Summary Report/Custom Report/Item Report) by selecting one report definition and
        then inputs criteria for the report query, selects one report style, and inputs the Dataset name for the report
        if user would like to save the report as a Dataset in Teamcenter. 
        """
        return cls.execute_soa_method(
            method_name='generateReport',
            library='Reports',
            service_date='2015_03',
            service_name='CrfReports',
            params={'inputs': inputs},
            response_cls=GenerateReportsResponse2,
        )

    @classmethod
    def generateReportAsync(cls, inputs: List[GenerateReportsCriteria2Async]) -> None:
        """
        Generates report asynchronously(Summary Report/Custom Report/Item Report) using the specified criteria and the
        selected report style sheet. The report can be displayed with the selected report style. The report style is
        used to define how to display the report result in UI to end user. The report can optionally be saved as a
        Dataset in the Teamcenter database. The generated report is saved in the FMS transient volume.
        """
        return cls.execute_soa_method(
            method_name='generateReportAsync',
            library='Reports',
            service_date='2015_03',
            service_name='CrfReports',
            params={'inputs': inputs},
            response_cls=None,
        )

    @classmethod
    def getPrintTemplates(cls, contextObjects: List[BusinessObject]) -> GetReportDefinitionsResponse:
        """
        This operation returns the list of associated ReportDefinition Objects to the current selected object.
        """
        return cls.execute_soa_method(
            method_name='getPrintTemplates',
            library='Reports',
            service_date='2015_03',
            service_name='CrfReports',
            params={'contextObjects': contextObjects},
            response_cls=GetReportDefinitionsResponse,
        )



from __future__ import annotations

from tcsoa.base import TcService


class SessionService(TcService):

    @classmethod
    def refreshPOMCachePerRequestDeprecated(cls, refresh: bool) -> bool:
        """
        By Default the service operations will retrieve property value data straight from the POM. When 'refresh' is
        set to true, a refresh will be done on business objects before getting any property data. This will update the
        POM with fresh data from the database. The refresh is only applied to business objects that are actually being
        returned by a service operation. This applies only to database objects, and is not applied to runtime objects. 
        This is applied to all subsequent service requests from the same client. If multiple clients are sharing the
        same Teamcenter server session the refresh POM state is applied per client. Setting this to true will have a
        performance impact but will grantee all property values returned are up-to-date.
        """
        return cls.execute_soa_method(
            method_name='refreshPOMCachePerRequestDeprecated',
            library='Internal-Core',
            service_date='2007_05',
            service_name='Session',
            params={'refresh': refresh},
            response_cls=bool,
        )



from tcsoa.gen.Internal_Visualization._2008_06.services import StructureManagementService as imp0
from tcsoa.gen.Internal_Visualization._2008_06.services import DataManagementService as imp1
from tcsoa.gen.Internal_Visualization._2010_09.services import DataManagementService as imp2
from tcsoa.gen.Internal_Visualization._2012_10.services import DataManagementService as imp3
from tcsoa.gen.Internal_Visualization._2010_09.services import StructureManagementService as imp4
from tcsoa.gen.Internal_Visualization._2011_12.services import StructureManagementService as imp5
from tcsoa.gen.Internal_Visualization._2010_04.services import DataManagementService as imp6
from tcsoa.gen.Internal_Visualization._2017_05.services import DataManagementService as imp7
from tcsoa.gen.Internal_Visualization._2018_11.services import StructureManagementService as imp8
from tcsoa.base import TcService


class StructureManagementService(TcService):
    areRecipesMergable = imp0.areRecipesMergable
    createBOMsFromRecipes = imp0.createBOMsFromRecipes
    createVisSC = imp4.createVisSC
    createVisSCsFromBOMs = imp0.createVisSCsFromBOMs
    expandPSFromOccurrenceList = imp0.expandPSFromOccurrenceList
    expandPSFromOccurrenceList2 = imp5.expandPSFromOccurrenceList
    getStructureIdFromRecipe = imp8.getStructureIdFromRecipe


class DataManagementService(TcService):
    authenticateUser = imp1.authenticateUser
    createLaunchFile = imp1.createLaunchFile
    createMarkup = imp1.createMarkup
    createSessionModel = imp1.createSessionModel
    createSnapshot3D = imp2.createSnapshot3D
    createSnapshot3D2 = imp3.createSnapshot3D
    createTwoDSnapshot = imp1.createTwoDSnapshot
    findProductViewsForNodes = imp2.findProductViewsForNodes
    gatherMarkups = imp6.gatherMarkups
    gatherSnapshot3DList = imp2.gatherSnapshot3DList
    getLatestFileReadTickets = imp1.getLatestFileReadTickets
    getMetaDataStamp = imp1.getMetaDataStamp
    getNodesPresentInProductView = imp2.getNodesPresentInProductView
    getSnapshot3DInfo = imp2.getSnapshot3DInfo
    getSnapshot3DInfo2 = imp7.getSnapshot3DInfo2
    saveSession = imp1.saveSession
    updateMarkup = imp1.updateMarkup
    updateSessionModel = imp1.updateSessionModel
    updateSnapshot3D = imp2.updateSnapshot3D
    updateSnapshot3DStructureFiles = imp2.updateSnapshot3DStructureFiles
    updateTwoDSnaphot = imp1.updateTwoDSnaphot
